import re

from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``ar_SA`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Saudi_Arabia

    .. |license_plate_en| replace::
        :meth:`license_plate_en()`
    """

    LICENSE_FORMAT_EN = "#### ???"
    LICENSE_FORMAT_AR = "? ? ? ####"

    PLATE_CHARS_EN = "ABDEGHJKLNRSTUVXZ"
    PLATE_CHARS_AR = "أبدعقهحكلنرسطوىصم"

    PLATE_MAP = {
        "A": "ا",
        "B": "ب",
        "D": "د",
        "E": "ع",
        "G": "ق",
        "H": "ه",
        "J": "ح",
        "K": "ك",
        "L": "ل",
        "N": "ن",
        "R": "ر",
        "S": "س",
        "T": "ط",
        "U": "و",
        "V": "ى",
        "X": "ص",
        "Z": "م",
        "0": "٠",
        "1": "١",
        "2": "٢",
        "3": "٣",
        "4": "٤",
        "5": "٥",
        "6": "٦",
        "7": "٧",
        "8": "٨",
        "9": "٩",
    }

    def license_plate_en(self) -> str:
        """Generate a license plate in Latin/Western characters."""
        return self.bothify(
            self.LICENSE_FORMAT_EN,
            letters=self.PLATE_CHARS_EN,
        )

    def license_plate_ar(self) -> str:
        """Generate a license plate in Arabic characters.

        This method first generates a license plate in Latin/Western characters
        using |license_plate_en|, and the result is translated internally to
        generate the Arabic counterpart which serves as this method's return
        value.
        """
        english_plate = self.license_plate_en()
        return self._translate_license_plate(english_plate)

    def _translate_license_plate(self, license_plate: str) -> str:
        nums = list(reversed(license_plate[0:4]))
        chars = list(license_plate[5:8])

        numerated = re.sub(
            r"\#",
            lambda x: self.PLATE_MAP[nums.pop()],
            self.LICENSE_FORMAT_AR,
        )
        ar_plate = re.sub(
            r"\?",
            lambda x: self.PLATE_MAP[chars.pop()],
            numerated,
        )

        return ar_plate

    def license_plate(self, ar: bool = True) -> str:
        return self.license_plate_ar() if ar else self.license_plate_en()



# -*- coding: utf-8 -*-

import re

from collections import OrderedDict

from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``es`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Chile

    """

    license_plate_old_format_first_letters = "ABCDFGHJKLPRSTVWXYZ"
    license_plate_old_format_second_letters = "ABCDFGHIJKLPRSTVWXYZ"
    license_plate_new_format_letters = "BCDFGHJKLPRSTVWXYZ"

    license_formats = OrderedDict(
        [
            ("{{license_plate_new}}", 0.70),
            ("{{license_plate_old}}", 0.20),
            ("{{license_plate_police}}", 0.05),
            ("{{license_plate_temporary}}", 0.04),
            ("{{license_plate_diplomatic}}", 0.01),
        ]
    )

    def license_plate_old(self) -> str:
        """Generate an old format license plate."""
        format = "-####"

        letters = "".join(
            (
                self.random_element(self.license_plate_old_format_first_letters),
                self.random_element(self.license_plate_old_format_second_letters),
            )
        )

        return self.numerify(letters + format)

    def license_plate_new(self) -> str:
        format = "????-##"

        temp = re.sub(r"\?", lambda x: self.random_element(self.license_plate_new_format_letters), format)
        return self.numerify(temp)

    def license_plate_police(self) -> str:
        formats = ("RP-####", "Z-####")
        return self.numerify(self.random_element(formats))

    def license_plate_temporary(self) -> str:
        format = "PR-###"
        return self.numerify(format)

    def license_plate_diplomatic(self) -> str:
        formats = ("CC-####", "CD-####")
        return self.numerify(self.random_element(formats))

    def license_plate(self) -> str:
        """Generate a license plate."""
        return self.numerify(self.generator.parse(self.random_element(self.license_formats)))



from faker.providers.job import Provider as JobProvider
from faker.providers.job.az_AZ import Provider as AzAzJobProvider
from faker.providers.job.cs_CZ import Provider as CsCzJobProvider
from faker.providers.job.de_DE import Provider as DeDeJobProvider
from faker.providers.job.el_GR import Provider as ElGrJobProvider
from faker.providers.job.es_ES import Provider as EsEsJobProvider
from faker.providers.job.fr_FR import Provider as FrFrJobProvider
from faker.providers.job.hu_HU import Provider as HuHuJobProvider
from faker.providers.job.hy_AM import Provider as HyAmJobProvider
from faker.providers.job.ja_JP import Provider as JaJpJobProvider
from faker.providers.job.ko_KR import Provider as KoKrJobProvider
from faker.providers.job.pt_BR import Provider as PtBrJobProvider
from faker.providers.job.pt_PT import Provider as PtPtJobProvider
from faker.providers.job.ro_RO import Provider as RoRoJobProvider
from faker.providers.job.sk_SK import Provider as SkSkJobProvider
from faker.providers.job.th_TH import Provider as ThThJobProvider
from faker.providers.job.tr_TR import Provider as TrTrJobProvider


class TestJobProvider:
    """Test job provider methods"""

    def test_job(self, faker, num_samples):
        for _ in range(num_samples):
            assert faker.job() in JobProvider.jobs


class TestCsCz:
    """Test cs_CZ job provider"""

    def test_job(self, faker, num_samples):
        for _ in range(num_samples):
            job = faker.job()
            assert isinstance(job, str)
            assert job in CsCzJobProvider.jobs


class TestAzAz:
    """Test az_AZ job provider"""

    def test_job(self, faker, num_samples):
        for _ in range(num_samples):
            assert faker.job() in AzAzJobProvider.jobs


class TestJaJp:
    """Test ja_JP job provider"""

    def test_job(self, faker, num_samples):
        for _ in range(num_samples):
            assert faker.job() in JaJpJobProvider.jobs


class TestKoKr:
    """Test ko_KR job provider"""

    def test_job(self, faker, num_samples):
        for _ in range(num_samples):
            assert faker.job() in KoKrJobProvider.jobs


class TestHuHu:
    """Test hu_HU job provider"""

    def test_job(self, faker, num_samples):
        for _ in range(num_samples):
            assert faker.job() in HuHuJobProvider.jobs


class TestHyAm:
    """Test hy_AM job provider"""

    def test_job(self, faker, num_samples):
        for _ in range(num_samples):
            assert faker.job() in HyAmJobProvider.jobs


class TestDeDe:
    """Test de_DE job provider"""

    def test_job(self, faker, num_samples):
        for _ in range(num_samples):
            assert faker.job() in DeDeJobProvider.jobs


class TestFrFr:
    """Test fr_FR job provider"""

    def test_job(self, faker, num_samples):
        for _ in range(num_samples):
            assert faker.job() in FrFrJobProvider.jobs


class TestElGr:
    """Test el_GR job provider"""

    def test_job(self, faker, num_samples):
        for _ in range(num_samples):
            assert faker.job() in ElGrJobProvider.jobs


class TestEsEs:
    """Test es job provider"""

    def test_job(self, faker, num_samples):
        for _ in range(num_samples):
            assert faker.job() in EsEsJobProvider.jobs


class TestPtPt:
    """Test pt_PT job provider"""

    def test_job(self, faker, num_samples):
        for _ in range(num_samples):
            assert faker.job() in PtPtJobProvider.jobs


class TestPtBr:
    """Test pt_BR job provider"""

    def test_job(self, faker, num_samples):
        for _ in range(num_samples):
            assert faker.job() in PtBrJobProvider.jobs


class TestSkSk:
    """Test sk_SK job provider"""

    def test_job(self, faker, num_samples):
        for _ in range(num_samples):
            job = faker.job()
            assert isinstance(job, str)
            assert job in SkSkJobProvider.jobs


class TestThTh:
    """Test th_TH job provider"""

    def test_job(self, faker, num_samples):
        assert faker.job() in ThThJobProvider.jobs


class TestTrTr:
    """Test tr_TR job provider"""

    def test_job(self, faker, num_samples):
        assert faker.job() in TrTrJobProvider.jobs


class TestRoRo:
    """Test tr_TR job provider"""

    def test_job(self, faker, num_samples):
        assert faker.job() in RoRoJobProvider.jobs



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for `nl_BE` locale.

    https://nl.wikipedia.org/wiki/Belgisch_kenteken
    """

    license_formats = (
        "???-###",  # 1973-2008
        "###-???",  # 2008-2010
        # New formats after 2010
        "1-???-###",
        "2-???-###",
    )



import datetime
import re

from string import ascii_uppercase
from typing import Tuple

from .. import BaseProvider, ElementsType

localized = True


class Provider(BaseProvider):
    """Implement default Passport provider for Faker."""

    passport_number_formats: ElementsType = ()

    def passport_dob(self) -> datetime.date:
        """Generate a datetime date of birth."""
        birthday = self.generator.date_of_birth()
        return birthday

    def passport_owner(self, gender: str = "X") -> Tuple[str, str]:
        """Generate a given_name and surname for a passport owner
        The ``gender`` argument is the gender marker of a passport owner, which is a one character string
        that is either male, female, or non-binary.
        """
        if gender == "M":
            given_name = self.generator.parse("{{first_name_male}}")
        elif gender == "F":
            given_name = self.generator.parse("{{first_name_female}}")
        else:
            given_name = self.generator.parse("{{first_name_nonbinary}}")

        surname = self.generator.parse("{{last_name}}")

        return given_name, surname

    def passport_number(self) -> str:
        """Generate a passport number by replacing tokens to be alphanumeric"""
        temp = re.sub(
            r"\?",
            lambda x: self.random_element(ascii_uppercase),
            self.random_element(self.passport_number_formats),
        )
        return self.numerify(temp)



from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        "{{last_name}} {{company_suffix}}",
        "{{last_name}} {{last_name}} {{company_suffix}}",
        "{{last_name}}",
    )

    company_suffixes = (
        "d.o.o.",
        "d.d.",
        "j.d.o.o.",
    )



from collections import OrderedDict

from .. import Provider as ColorProvider

localized = True


class Provider(ColorProvider):
    """
    Implement color provider for ``da_DK`` locale. Source: https://tools.hopetrip.com.hk/web/colorcode/index-da-1.html
    """

    all_colors = OrderedDict(
        (
            ("Baker-Miller lyserød", "#FF91AF"),
            ("Lysegul (Crayola)", "#FFAA1D"),
            ("Rav", "#FFBF00"),
            ("Rav (SAE / ECE)", "#FF7E00"),
            ("Atomisk mandarin", "#FF9966"),
            ("Bisque", "#FFE4C4"),
            ("Candy apple rød", "#FF0800"),
            ("Babypudder", "#FEFEFA"),
            ("Coquelicot", "#FF3800"),
            ("Blancheret mandel", "#FFEBCD"),
            ("Strålende rose", "#FF55A3"),
            ("Bittersød", "#FE6F5E"),
            ("Cadmium gul", "#FFF600"),
            ("Kanariefarvet gul", "#FFEF00"),
            ("Nellike lyserød", "#FFA6C9"),
            ("Kanariefugl", "#FFFF99"),
            ("Kirsebærblomst lyserød", "#FFB7C5"),
            ("Koral", "#FF7F50"),
            ("Kinesisk gul", "#FFB200"),
            ("Krom gul", "#FFA700"),
            ("Cybergult", "#FFD300"),
            ("Dyb lyserød", "#FF1493"),
            ("Aureolin", "#FDEE00"),
            ("Bananmani", "#FAE7B5"),
            ("Candyfloss", "#FFBCD9"),
            ("Fløde", "#FFFDD0"),
            ("Kosmisk latte", "#FFF8E7"),
            ("Cornsilk", "#FFF8DC"),
            ("Mørk orange", "#FF8C00"),
            ("Dyb safran", "#FF9933"),
            ("Blomsterhvid", "#FFFAF0"),
            ("Brændende rose", "#FF5470"),
            ("Fransk lyserød", "#FD6C9E"),
            ("Fuchsia", "#FF00FF"),
            ("Fransk fuchsia", "#FD3F92"),
            ("Antik hvid", "#FAEBD7"),
            ("Blond", "#FAF0BE"),
            ("Brink pink", "#FB607F"),
            ("Abrikos", "#FBCEB1"),
            ("Majs", "#FBEC5D"),
            ("Koralrosa", "#F88379"),
            ("Kultiveret", "#F5F5F5"),
            ("Cameo pink", "#EFBBCC"),
            ("Champagne", "#F7E7CE"),
            ("Flickr Pink", "#FB0081"),
            ("Dyb champagne", "#FAD6A5"),
            ("Congo pink", "#F88379"),
            ("Champagne lyserød", "#F1DDCF"),
            ("Baby lyserød", "#F4C2C2"),
            ("Beige", "#F5F5DC"),
            ("Fransk rose", "#F64A8A"),
            ("Cyclamen", "#F56FA1"),
            ("Azure (X11 / webfarve)", "#F0FFFF"),
            ("Alice blå", "#F0F8FF"),
            ("Mode fuchsia", "#F400A1"),
            ("Mandel", "#EFDECD"),
            ("Æggeskal", "#F0EAD6"),
            ("Hollandsk hvid", "#EFDFBB"),
            ("Amaranth pink", "#F19CBB"),
            ("Buff", "#F0DC82"),
            ("Ørkensand", "#EDC9AF"),
            ("Cadmium orange", "#ED872D"),
            ("Arylid gul", "#E9D66B"),
            ("Brandopal", "#E95C4B"),
            ("Alabaster", "#EDEAE0"),
            ("Gulerod orange", "#ED9121"),
            ("Hør", "#EEDC82"),
            ("Brændt sienna", "#E97451"),
            ("Cadmium rød", "#E30022"),
            ("Mørk laks", "#E9967A"),
            ("Flamme", "#E25822"),
            ("Knogle", "#E3DAC9"),
            ("Amaranth", "#E52B50"),
            ("Forfrysninger", "#E936A7"),
            ("Fulvous", "#E48400"),
            ("Charm lyserød", "#E68FAC"),
            ("Candy pink", "#E4717A"),
            ("Fawn", "#E5AA70"),
            ("Citrin", "#E4D00A"),
            ("Cinnabar", "#E34234"),
            ("CG rød", "#E03C31"),
            ("Crimson", "#DC143C"),
            ("Jorden gul", "#E1A95F"),
            ("Kina lyserød", "#DE6FA1"),
            ("Rødme", "#DE5D83"),
            ("Chartreuse (traditionel)", "#DFFF00"),
            ("Burlywood", "#DEB887"),
            ("Cerise", "#DE3163"),
            ("Barbie Pink", "#DA1884"),
            ("Fandango pink", "#DE5285"),
            ("Dyb cerise", "#DA3287"),
            ("Chokolade (web)", "#D2691E"),
            ("Lys lilla", "#D891EF"),
            ("Dogwood rose", "#D71868"),
            ("Carmine (M&P)", "#D70040"),
            ("Kobber (Crayola)", "#DA8A67"),
            ("Amaranth rød", "#D3212D"),
            ("Fransk mauve", "#D473D4"),
            ("Kakaobrun", "#D2691E"),
            ("Arktisk kalk", "#D0FF14"),
            ("Brandbil rød", "#CE2029"),
            ("Cedertræskiste", "#C95A49"),
            ("Kamel", "#C19A6B"),
            ("Brændt orange", "#CC5500"),
            ("Fransk hindbær", "#C72C48"),
            ("Aero blå", "#C9FFE5"),
            ("Bitter citron", "#CAE00D"),
            ("Kanel Satin", "#CD607E"),
            ("Lys rødbrun", "#C32148"),
            ("Ørken", "#C19A6B"),
            ("Fuchsia lilla", "#CC397B"),
            ("Engelsk vermillion", "#CC474B"),
            ("Antik messing", "#CD9575"),
            ("Bronze", "#CD7F32"),
            ("Elektrisk kalk", "#CCFF00"),
            ("Fuchsia steg", "#C74375"),
            ("Murstensrød", "#CB4154"),
            ("Legeret orange", "#C46210"),
            ("Kobberrød", "#CB6D51"),
            ("Kardinal", "#C41E3A"),
            ("Bitter kalk", "#BFFF00"),
            ("Engelsk lavendel", "#B48395"),
            ("Celeste", "#B2FFFF"),
            ("Mørk kaki", "#BDB76B"),
            ("Brak", "#C19A6B"),
            ("Fuchsia (Crayola)", "#C154C1"),
            ("Ecru", "#C2B280"),
            ("Bittersød glimmer", "#BF4F51"),
            ("Lyseblå", "#BCD4E6"),
            ("Sorte skygger", "#BFAFB2"),
            ("Elektrisk lilla", "#BF00FF"),
            ("Byzantinsk", "#BD33A4"),
            ("Columbia Blue", "#B9D9EB"),
            ("Mørk guldrør", "#B8860B"),
            ("Kobber", "#B87333"),
            ("Dyb kastanje", "#B94E48"),
            ("Carnelian", "#B31B1B"),
            ("Afrikansk violet", "#B284BE"),
            ("Fandango", "#B53389"),
            ("Askegrå", "#B2BEB5"),
            ("Kobber øre", "#AD6F69"),
            ("Auburn", "#A52A2A"),
            ("Celadon", "#ACE1AF"),
            ("Syregrøn", "#B0BF1A"),
            ("Amaranth lilla", "#AB274F"),
            ("brunt sukker", "#AF6E4D"),
            ("Blå klokke", "#A2A2D0"),
            ("Firebrick", "#B22222"),
            ("Kinesisk rød", "#AA381E"),
            ("Engelsk rød", "#AB4B52"),
            ("Café au lait", "#A67B5B"),
            ("Kadetblå (Crayola)", "#A9B2C3"),
            ("Snestorm blå", "#ACE5EE"),
            ("Kina steg", "#A8516E"),
            ("Fransk beige", "#A67B5B"),
            ("Blast-off bronze", "#A57164"),
            ("Flirt", "#A2006D"),
            ("Android grøn", "#A4C639"),
            ("Cambridge blå", "#A3C1AD"),
            ("Babyblå øjne", "#A1CAF1"),
            ("Amaranth (M&P)", "#9F2B68"),
            ("Cinereous", "#98817B"),
            ("Kedelig", "#967117"),
            ("Citron", "#9FA91F"),
            ("Bæver", "#9F8170"),
            ("Crimson (UA)", "#9E1B32"),
            ("Kadetgrå", "#91A3B0"),
            ("Brunbrun", "#A17A74"),
            ("Stor dukkert o’ruby", "#9C2542"),
            ("Fransk kalk", "#9EFD38"),
            ("Ametyst", "#9966CC"),
            ("Kobber rose", "#996666"),
            ("Eton blå", "#96C8A2"),
            ("Carmine", "#960018"),
            ("Bistre brun", "#967117"),
            ("Mørk orkidé", "#9932CC"),
            ("Mørk violet", "#9400D3"),
            ("Artiskok", "#8F9779"),
            ("kastanje", "#954535"),
            ("Antik fuchsia", "#915C83"),
            ("Mørk havgrøn", "#8FBC8F"),
            ("Baby Blå", "#89CFF0"),
            ("Cool grå", "#8C92AC"),
            ("Mørk magenta", "#8B008B"),
            ("Cordovan", "#893F45"),
            ("Mørk himmelblå", "#8CBED6"),
            ("Æblegrøn", "#8DB600"),
            ("Brun", "#88540B"),
            ("Asparges", "#87A96B"),
            ("Brandy", "#87413F"),
            ("Elektrisk violet", "#8F00FF"),
            ("Blåviolet", "#8A2BE2"),
            ("Brændt umber", "#8A3324"),
            ("Mørkerød", "#8B0000"),
            ("Fransk violet", "#8806CE"),
            ("Fransk lilla", "#86608E"),
            ("Fuzzy Wuzzy", "#87421F"),
            ("Antik rubin", "#841B2D"),
            ("Slagskib grå", "#848482"),
            ("Kinesisk violet", "#856088"),
            ("Aero", "#7CB9E8"),
            ("Coyote brun", "#81613C"),
            ("Byzantium", "#702963"),
            ("Chokolade (traditionel)", "#7B3F00"),
            ("Akvamarin", "#7FFFD4"),
            ("Fransk bistre", "#856D4D"),
            ("Bourgogne", "#800020"),
            ("Dyb taupe", "#7E5E60"),
            ("Falu rød", "#801818"),
            ("Laderød", "#7C0A02"),
            ("Claret", "#7F1734"),
            ("Fransk himmelblå", "#77B5FE"),
            ("Elektrisk blå", "#7DF9FF"),
            ("Bole", "#79443B"),
            ("Kaffe", "#6F4E37"),
            ("Luftoverlegenhed blå", "#72A0C1"),
            ("Blåviolet (Crayola)", "#7366BD"),
            ("Knoppegrøn", "#7BB661"),
            ("Catawba", "#703642"),
            ("Kornblomst blå", "#6495ED"),
            ("Elektrisk indigo", "#6F00FF"),
            ("Eminence", "#6C3082"),
            ("Blågrå", "#6699CC"),
            ("Mark trist", "#6C541E"),
            ("Blodrød", "#660000"),
            ("Cerulean frost", "#6D9BC3"),
            ("Dim grå", "#696969"),
            ("Lyse-grøn", "#66FF00"),
            ("Kadetblå", "#5F9EA0"),
            ("Mørkeblå-grå", "#666699"),
            ("Cyber ​​drue", "#58427C"),
            ("Caput mortuum", "#592720"),
            ("Aubergine", "#614051"),
            ("Mørkt byzantium", "#5D3954"),
            ("Antik bronze", "#665D1E"),
            ("Skovgrøn (Crayola)", "#5FA777"),
            ("Mørkebrun", "#654321"),
            ("Avocado", "#568203"),
            ("Blå bukser", "#5DADEC"),
            ("Mørk elektrisk blå", "#536878"),
            ("Mørk lever (heste)", "#543D37"),
            ("Café noir", "#4B3621"),
            ("Smaragd", "#50C878"),
            ("Carolina blå", "#56A0D3"),
            ("Kadet", "#536872"),
            ("Mørk lever", "#534B4F"),
            ("Engelsk violet", "#563C5C"),
            ("Mørk olivengrøn", "#556B2F"),
            ("Sort koral", "#54626F"),
            ("Blå derfra", "#5072A7"),
            ("Ibenholt", "#555D50"),
            ("Davy er grå", "#555555"),
            ("Militærgrøn", "#4B5320"),
            ("Feldgrau", "#4D5D53"),
            ("Fern grøn", "#4F7942"),
            ("Mørk mosgrøn", "#4A5D23"),
            ("Mørk lava", "#483C32"),
            ("Blåviolet (farvehjul)", "#4D1A7F"),
            ("Deep Space Sparkle", "#4A646C"),
            ("Mørk skiferblå", "#483D8B"),
            ("Sort bønne", "#3D0C02"),
            ("Bistre", "#3D2B1F"),
            ("Sort oliven", "#3B3C36"),
            ("Bluetiful", "#3C69E7"),
            ("B'dazzled blå", "#2E5894"),
            ("Trækul", "#36454F"),
            ("Cerulean blå", "#2A52BE"),
            ("Kosmisk kobolt", "#2E2D88"),
            ("Celadon grøn", "#2F847C"),
            ("Sort kaffe", "#3B2F2F"),
            ("Amazon", "#3B7A57"),
            ("Mørk sienna", "#3C1414"),
            ("Blå (pigment)", "#333399"),
            ("Mørk skifergrå", "#2F4F4F"),
            ("Mørk kornblomst", "#26428B"),
            ("Bleu de France", "#318CE7"),
            ("Mørke lilla", "#301934"),
            ("Keltisk blå", "#246BCE"),
            ("Charleston grøn", "#232B2B"),
            ("Dodger blå", "#1E90FF"),
            ("Blågrøn (farvehjul)", "#064E40"),
            ("Denim", "#1560BD"),
            ("Eerie sort", "#1B1B1B"),
            ("Denimblå", "#2243B6"),
            ("Blå (Crayola)", "#1F75FE"),
            ("Flickr Blue", "#0063dc"),
            ("Skovgrøn (web)", "#228B22"),
            ("Sort chokolade", "#1B1811"),
            ("Engelsk grøn", "#1B4D3E"),
            ("Brunswick grøn", "#1B4D3E"),
            ("Cerulean (Crayola)", "#1DACD6"),
            ("Fluorescerende blå", "#15F4EE"),
            ("Lys marineblå", "#1974D2"),
            ("Mørk jungle grøn", "#1A2421"),
            ("Mørk forår grøn", "#177245"),
            ("Blå (RYB)", "#0247FE"),
            ("Egyptisk blå", "#1034A6"),
            ("Blå safir", "#126180"),
            ("Blågrøn", "#0D98BA"),
            ("Mørk pastelgrøn", "#03C03C"),
        )
    )

    safe_colors = (
        "sort",
        "rødbrun",
        "grøn",
        "mørkeblå",
        "oliven",
        "lilla",
        "blågrøn",
        "lime",
        "blå",
        "sølv",
        "grå",
        "gul",
        "pink",
        "turkis",
        "hvid",
    )



from .. import Provider as BaseProvider


# Reference:
# http://zetawiki.com/wiki/%ED%95%9C%EA%B5%AD%ED%91%9C%EC%A4%80%EC%A7%81%EC%97%85%EB%B6%84%EB%A5%98_%ED%91%9C
# on 2017-08-30
class Provider(BaseProvider):
    jobs = [
        "의회의원/고위공무원 및 공공단체임원",
        "기업고위임원",
        "정부행정 관리자",
        "경영지원 관리자",
        "기타 행정 및 경영지원 관리자",
        "연구 관리자",
        "교육 관리자",
        "법률/경찰/소방 및 교도 관리자",
        "보험 및 금융 관리자",
        "보건의료관련 관리자",
        "사회복지관련 관리자",
        "문화/예술/디자인 및 영상관련 관리자",
        "정보통신관련 관리자",
        "기타 전문서비스 관리자",
        "건설 및 광업 관련 관리자",
        "전기/가스 및 수도 관련 관리자",
        "제품 생산관련 관리자",
        "기타 건설/전기 및 생산 관련 관리자",
        "영업 및 판매 관련 관리자",
        "운송관련 관리자",
        "숙박/여행/오락 및 스포츠 관련 관리자",
        "음식서비스관련 관리자",
        "환경/청소 및 경비 관련 관리자",
        "기타 판매 및 고객 서비스 관리자",
        "생명과학 연구원",
        "자연과학 연구원",
        "인문과학 연구원",
        "사회과학 연구원",
        "생명과학 시험원",
        "농림어업관련 시험원",
        "자연과학 시험원",
        "컴퓨터 하드웨어 기술자 및 연구원",
        "통신공학 기술자 및 연구원",
        "컴퓨터시스템 설계 및 분석가",
        "시스템 소프트웨어 개발자",
        "응용 소프트웨어 개발자",
        "데이터베이스 개발자",
        "네트워크시스템 개발자",
        "컴퓨터 보안 전문가",
        "웹 및 멀티미디어 기획자",
        "웹 개발자",
        "정보 시스템 운영자",
        "통신 및 방송송출 장비 기사",
        "건축가 및 건축공학 기술자",
        "토목공학 기술자",
        "조경 기술자",
        "도시 및 교통설계 전문가",
        "측량 및 지리정보 전문가",
        "건설자재 시험원",
        "화학공학 기술자 및 연구원",
        "화학공학 시험원",
        "금속 / 재료공학 연구원 및 기술자",
        "금속 / 재료공학 시험원",
        "환경공학 기술자 및 연구원",
        "환경공학 시험원",
        "전기공학 기술자 및 연구원",
        "전자공학 기술자 및 연구원",
        "기계공학 기술자 및 연구원",
        "전기/전자 및 기계 공학 시험원",
        "산업안전 및 위험 관리원",
        "보건위생 및 환경 검사원",
        "비파괴 검사원",
        "항공기 조종사",
        "선장/항해사 및 도선사",
        "관제사",
        "식품공학 기술자 및 연구원",
        "섬유공학 기술자 및 연구원",
        "가스/에너지 기술자 및 연구원",
        "소방공학 기술자 및 연구원",
        "식품/섬유 공학 및 에너지 시험원",
        "캐드원",
        "기타 공학관련 기술자 및 시험원",
        "전문 의사",
        "일반 의사",
        "한의사",
        "치과 의사",
        "수의사",
        "약사 및 한약사",
        "간호사",
        "영양사",
        "임상병리사",
        "방사선사",
        "치과기공사",
        "치과위생사",
        "의지보조기기사",
        "물리 및 작업 치료사",
        "임상 심리사 및 기타 치료사",
        "응급구조사",
        "위생사",
        "안경사",
        "의무기록사",
        "간호조무사",
        "안마사",
        "사회복지사",
        "보육 교사",
        "직업상담사 및 취업 알선원",
        "상담 전문가 및 청소년 지도사",
        "시민 단체 활동가",
        "기타 사회복지관련 종사원",
        "성직자",
        "기타 종교관련 종사자",
        "대학 교수",
        "대학 시간강사",
        "중/고등학교 교사",
        "초등학교 교사",
        "특수교육 교사",
        "유치원 교사",
        "문리 및 어학 강사",
        "컴퓨터 강사",
        "기술 및 기능계 강사",
        "예능 강사",
        "학습지 및 방문 교사",
        "기타 문리/기술 및 예능 강사",
        "장학관/연구관 및 교육 관련 전문가",
        "대학 교육조교",
        "보조 교사 및 기타 교사",
        "판사 및 검사",
        "변호사",
        "법무사 및 집행관",
        "변리사",
        "정부 및 공공 행정 전문가",
        "인사 및 노사 관련 전문가",
        "회계사",
        "세무사",
        "관세사",
        "경영 및 진단 전문가",
        "투자 및 신용 분석가",
        "자산 운용가",
        "보험 및 금융 상품 개발자",
        "증권 및 외환 딜러",
        "손해사정인",
        "기타 금융 및 보험 관련 전문가",
        "상품기획 전문가",
        "여행상품 개발자",
        "광고 및 홍보 전문가",
        "조사 전문가",
        "행사기획자",
        "감정평가 전문가",
        "해외 영업원",
        "기술 영업원",
        "상품중개인 및 경매사",
        "부동산 컨설턴트 및 중개인",
        "기타 기술영업 및 중개 관련 종사자",
        "작가 및 관련 전문가",
        "번역가",
        "통역가",
        "기자 및 논설위원",
        "출판물 전문가",
        "큐레이터 및 문화재 보존원",
        "사서 및 기록물관리사",
        "감독 및 기술감독",
        "배우 및 모델",
        "아나운서 및 리포터",
        "촬영기사",
        "음향 및 녹음 기사",
        "영상/녹화 및 편집 기사",
        "조명기사 및 영사기사",
        "기타 연극/영화 및 영상 관련 종사자",
        "화가 및 조각가",
        "사진기자 및 사진가",
        "만화가 및 만화영화 작가",
        "국악 및 전통예능인",
        "지휘자/작곡가 및 연주가",
        "가수 및 성악가",
        "무용가 및 안무가",
        "제품 디자이너",
        "패션 디자이너",
        "실내장식 디자이너",
        "시각 디자이너",
        "웹 및 멀티미디어 디자이너",
        "경기감독 및 코치",
        "직업 운동선수",
        "경기심판 및 경기기록원",
        "스포츠 및 레크레이션 강사",
        "기타 스포츠 및 레크레이션 관련 전문가",
        "연예인 및 스포츠 매니저",
        "마술사 및 기타 문화/ 예술 관련 종사자",
        "조세행정 사무원",
        "관세행정 사무원",
        "병무행정 사무원",
        "국가/지방 및 공공행정 사무원",
        "기획 및 마케팅 사무원",
        "인사 및 교육/훈련 사무원",
        "자재관리 사무원",
        "생산 및 품질 관리 사무원",
        "무역 사무원",
        "운송 사무원",
        "총무 사무원",
        "회계 사무원",
        "경리 사무원",
        "비서",
        "전산 자료 입력원 및 사무 보조원",
        "출납창구 사무원",
        "보험 심사원 및 사무원",
        "금융관련 사무원",
        "신용 추심원",
        "법률관련 사무원",
        "감사 사무원",
        "통계관련 사무원",
        "여행 사무원",
        "안내 / 접수 사무원 및 전화교환원",
        "고객 상담 및 모니터 요원",
        "기타 사무원",
        "경찰관",
        "소방관",
        "소년보호관 및 교도관",
        "경호원",
        "청원 경찰",
        "무인 경비원",
        "기타 경호 및 보안 관련 종사원",
        "간병인",
        "기타 의료/복지 관련 서비스 종사원",
        "이용사",
        "미용사",
        "피부미용 및 체형관리사",
        "메이크업 아티스트 및 분장사",
        "애완동물 미용사",
        "기타 미용관련 서비스 종사원",
        "결혼 상담원 및 웨딩플래너",
        "혼례 종사원",
        "장례 상담원 및 장례 지도사",
        "기타 이미용/예식 및 의료보조 서비스 종사원",
        "항공기 객실승무원",
        "선박 및 열차 객실승무원",
        "여행 및 관광통역 안내원",
        "숙박시설 서비스원",
        "오락시설 서비스원",
        "기타 여가 및 스포츠 관련 종사원",
        "한식 주방장 및 조리사",
        "중식 주방장 및 조리사",
        "양식 주방장 및 조리사",
        "일식 주방장 및 조리사",
        "기타 주방장 및 조리사",
        "바텐더",
        "웨이터",
        "기타 음식서비스 종사원",
        "자동차 영업원",
        "제품 및 광고 영업원",
        "보험 설계사 및 간접투자증권 판매인",
        "상점 판매원",
        "매표원 및 복권 판매원",
        "매장계산원 및 요금정산원",
        "상품 대여원",
        "방문 판매원",
        "통신서비스판매원",
        "텔레마케터",
        "인터넷 판매원",
        "노점 및 이동 판매원",
        "홍보 도우미 및 판촉원",
        "곡식작물 재배원",
        "채소 및 특용작물 재배원",
        "과수작물 재배원",
        "원예작물 재배원",
        "조경원",
        "낙농업관련 종사원",
        "가축 사육 종사원",
        "기타 사육관련 종사원",
        "조림/영림 및 벌목원",
        "임산물채취 및 기타 임업 관련 종사원",
        "양식원",
        "어부 및 해녀",
        "제빵원 및 제과원",
        "떡제조원",
        "정육원 및 도축원",
        "식품 및 담배 등급원",
        "김치 및 밑반찬 제조 종사원",
        "기타 식품가공관련 종사원",
        "패턴사",
        "재단사",
        "재봉사",
        "제화원",
        "기타 섬유 및 가죽 관련 기능 종사원",
        "한복 제조원",
        "양장 및 양복 제조원",
        "모피 및 가죽의복 제조원",
        "의복/가죽 및 모피 수선원",
        "기타 의복 제조원",
        "목제품 제조관련 종사원",
        "가구 제조 및 수리원",
        "악기제조 및 조율사",
        "간판 제작 및 설치원",
        "금형원",
        "주조원",
        "단조원",
        "제관원",
        "판금원",
        "용접원",
        "자동차 정비원",
        "항공기 정비원",
        "선박 정비원",
        "철도 기관차 및 전동차 정비원",
        "기타 운송장비 정비원",
        "공업기계 설치 및 정비원",
        "승강기 설치 및 정비원",
        "물품 이동 장비 설치 및 정비원",
        "냉동/냉장 /공조기 설치 및 정비원",
        "보일러 설치 및 정비원",
        "건설 및 광업기계 설치 및 정비원",
        "농업용 및 기타 기계장비 설치 및 정비원",
        "가전제품 설치 및 수리원",
        "기타 전기/전자기기 설치 및 수리원",
        "산업전공",
        "내선전공",
        "외선전공",
        "강구조물 가공원 및 건립원",
        "경량 철골공",
        "철근공",
        "콘크리트공",
        "건축 석공",
        "건축 목공",
        "조적공 및 석재 부설원",
        "기타 건설관련 기능 종사원",
        "미장공",
        "방수공",
        "단열공",
        "바닥재 시공원",
        "도배공 및 유리 부착원",
        "건축 도장공",
        "섀시 조립 및 설치원",
        "기타 건축마감관련 기능 종사원",
        "광원/채석원 및 석재 절단원",
        "철로 설치 및 보수원",
        "기타 채굴 및 토목 관련 종사자",
        "영상 및 관련 장비 설치 및 수리원",
        "통신 및 관련 장비 설치 및 수리원",
        "통신/방송 및 인터넷 케이블 설치 및 수리원",
        "공예원",
        "귀금속 및 보석 세공원",
        "건설 배관공",
        "공업 배관공",
        "기타 배관공",
        "배관 세정원 및 방역원",
        "기타 기능관련 종사원",
        "제분 및 도정 관련 기계 조작원",
        "곡물가공제품 기계 조작원",
        "육류/어패류 및 낙농품 가공 기계조작원",
        "과실 및 채소 관련 기계조작원",
        "음료 제조관련 기계 조작원",
        "기타 식품가공관련 기계조작원",
        "섬유제조 기계조작원",
        "표백 및 염색 관련 조작원",
        "직조기 및 편직기 조작원",
        "신발제조기 조작원 및 조립원",
        "기타 직물 및 신발 관련 기계조작원 및 조립원",
        "세탁관련 기계조작원",
        "석유 및 천연가스제조 관련 제어장치 조작원",
        "화학물 가공장치 조작원",
        "기타 석유 및 화학물 가공장치 조작원",
        "화학제품 생산기 조작원",
        "타이어 및 고무제품 생산기 조작원",
        "플라스틱제품 생산기 조작원",
        "고무 및 플라스틱 제품 조립원",
        "주조기 조작원",
        "단조기 조작원",
        "용접기 조작원",
        "금속가공관련 제어장치 조작원",
        "금속가공 기계조작원",
        "제관기 조작원",
        "판금기 조작원",
        "도장기 조작원",
        "도금 및 금속분무기 조작원",
        "유리제조 및 가공기 조작원",
        "점토제품 생산기 조작원",
        "시멘트 및 광물제품 제조기 조작원",
        "광석 및 석제품 가공기 조작원",
        "기타 비금속제품관련 생산기 조작원",
        "금속공작기계 조작원",
        "냉/난방 관련 설비 조작원",
        "자동조립라인 및 산업용 로봇 조작원",
        "자동차 조립원",
        "자동차 부분품 조립원",
        "운송장비 조립원",
        "일반기계 조립원",
        "금속기계부품 조립원",
        "발전 및 배전장치 조작원",
        "전기 및 전자 설비 조작원",
        "전기 부품 및 제품제조 기계조작원",
        "전자 부품 및 제품 제조 기계조작원",
        "전기/전자 부품 및 제품 조립원",
        "철도 및 전동차 기관사",
        "화물열차 차장 및 관련 종사원",
        "택시 운전원",
        "버스 운전원",
        "화물차 및 특수차 운전원",
        "기타 자동차 운전원",
        "물품이동 장비 조작원",
        "건설 및 채굴 기계 운전원",
        "선박 갑판승무원 및 관련 종사원",
        "상/하수도 처리장치 조작원",
        "재활용 처리 및 소각로 조작원",
        "목재 가공관련 기계 조작원",
        "가구조립원",
        "펄프 및 종이 제조장치 조작원",
        "종이제품 생산기 조작원",
        "기타 목재 및 종이 관련 기계조작원",
        "인쇄기 조작원",
        "사진인화 및 현상기 조작원",
        "기타 제조관련 기계 조작원",
        "건설 및 광업 단순 종사원",
        "하역 및 적재 단순 종사원",
        "우편물 집배원",
        "택배원",
        "음식 배달원",
        "기타 배달원",
        "제조관련 단순 종사원",
        "청소원",
        "환경 미화원 및 재활용품 수거원",
        "경비원",
        "검표원",
        "가사 도우미",
        "육아 도우미",
        "패스트푸드원",
        "주방 보조원",
        "주유원",
        "기타 판매관련 단순 종사원",
        "농림어업관련 단순 종사원",
        "계기 검침원 및 가스점검원",
        "수금원",
        "주차 관리원 및 안내원",
        "구두 미화원",
        "세탁원 및 다림질원",
        "기타 서비스관련 단순 종사원",
        "영관급 이상",
        "위관급",
        "장기 부사관 및 준위",
    ]



from collections import OrderedDict
from typing import List, Tuple

from ..es import Provider as AddressProvider


class Provider(AddressProvider):
    provinces = {
        "CABA": "Ciudad Autónoma de Buenos Aires",
        "BA": "Buenos Aires",
        "CA": "Catamarca",
        "CH": "Chaco",
        "CT": "Chubut",
        "CB": "Córdoba",
        "CR": "Corrientes",
        "ER": "Entre Ríos",
        "FO": "Formosa",
        "JY": "Jujuy",
        "LP": "La Pampa",
        "LR": "La Rioja",
        "MZ": "Mendoza",
        "MI": "Misiones",
        "NQN": "Neuquén",
        "RN": "Río Negro",
        "SA": "Salta",
        "SJ": "San Juan",
        "SL": "San Luis",
        "SC": "Santa Cruz",
        "SF": "Santa Fe",
        "SE": "Santiago del Estero",
        "TF": "Tierra del Fuego",
        "TU": "Tucumán",
    }

    municipalities: List[Tuple[str, str, str]] = [
        ("1004", "Constitución", "CABA"),
        ("1900", "La Plata", "BA"),
        ("7600", "Mar del Plata", "BA"),
        ("8000", "Bahía Blanca", "BA"),
        ("4700", "San Ferando del Valle de Catamarca", "CA"),
        ("3500", "Resistencia", "CH"),
        ("9103", "Rawson", "CT"),
        ("9000", "Comodoro Rivadavia", "CT"),
        ("5000", "Córdoba", "CB"),
        ("3400", "Corrientes", "CR"),
        ("3100", "Paraná", "ER"),
        ("3600", "Formosa", "FO"),
        ("4600", "San Salvador de Jujuy", "JY"),
        ("6300", "Santa Rosa", "LP"),
        ("5300", "La Rioja", "LR"),
        ("5360", "Chilecito", "LR"),
        ("5500", "Mendoza", "MZ"),
        ("3300", "Posadas", "MI"),
        ("8300", "Neuquén", "NQN"),
        ("8500", "Viedma", "RN"),
        ("4400", "Salta", "SA"),
        ("5400", "San Juan", "SJ"),
        ("5700", "San Luis", "SL"),
        ("5881", "Merlo", "SL"),
        ("9400", "Río Gallegos", "SC"),
        ("3000", "Santa Fe", "SF"),
        ("2000", "Rosario", "SF"),
        ("4200", "Santiago del Estero", "SE"),
        ("9410", "Ushuaia", "TF"),
        ("4000", "San Miguel de Tucumán", "TU"),
    ]

    street_prefixes = OrderedDict(
        [
            ("Calle", 0.2),
            ("Avenida", 0.2),
            ("Av.", 0.2),
            ("Diagonal", 0.2),
            ("Diag.", 0.05),
            ("Camino", 0.05),
            ("Boulevard", 0.05),
            ("Blv.", 0.05),
        ]
    )
    street_suffixes = ["A", "B", "Bis"]

    street_proceres = (
        "San Martin",
        "Belgrano",
        "Saavedra",
        "Rivadavia",
        "Güemes",
        "G. Brown",
        "J.B. Alberdi",
        "J.M. de Rosas",
        "J.J. Castelli",
        "Mitre",
        "Alem",
        "Alvear",
        "Malvinas Argentinas",
        "Pte. Perón",
        "Omar Nuñez",
    )
    street_name_formats = OrderedDict(
        [
            ("{{street_prefix}} %", 0.2),
            ("{{street_prefix}} {{street_municipality}}", 0.2),
            ("{{street_prefix}} {{street_province}}", 0.2),
            ("{{street_prefix}} {{street_procer}}", 0.2),
            ("{{street_prefix}} 1## {{street_suffix}}", 0.02),
        ]
    )
    building_number_formats = OrderedDict(
        [
            ("%%", 0.2),
            ("%%#", 0.2),
            ("%#%", 0.2),
            ("%#%#", 0.2),
        ]
    )
    secondary_address_formats = [
        "Piso % Dto. %",
        "Dto. %",
        "Torre % Dto. %",
        "Local %!",
        "Oficina %!",
    ]
    postcode_formats = ["{{municipality_code}}####"]

    def provinces_code(self) -> str:
        """
        :example: "BA"
        """
        return self.random_element(self.provinces.keys())

    def province(self) -> str:
        """
        :example: "Buenos Aires"
        """
        return self.random_element(list(self.provinces.values()))

    administrative_unit = province

    def municipality_code(self) -> str:
        """
        :example: "1900"
        """
        return self.random_element(self.municipalities)[0]  # type: ignore

    def municipality(self) -> str:
        """
        :example: "La Plata"
        """
        return self.random_element(self.municipalities)[1]  # type: ignore

    city = municipality

    def street_prefix(self) -> str:
        """
        :example: "Calle"
        """
        return self.random_element(self.street_prefixes)

    def street_procer(self) -> str:
        """
        :example: "Belgrano"
        """
        return self.random_element(self.street_proceres)

    def street_municipality(self) -> str:
        """
        :example: "La Plata"
        """
        return self.random_element(self.municipalities)[1]

    def street_province(self) -> str:
        """
        :example: "San Juan"
        """
        return self.random_element(list(self.provinces.values()))

    def street_suffix(self) -> str:
        """
        :example: "Sur"
        """
        return self.generator.parse(self.random_element(self.street_suffixes))

    def street_name(self) -> str:
        """
        :example: "Calle 1"
        """
        pattern: str = self.random_element(self.street_name_formats)
        return self.numerify(self.generator.parse(pattern))

    def building_number(self) -> str:
        """
        :example: "23"
        """
        return self.numerify(self.generator.parse(self.random_element(self.building_number_formats)))

    def secondary_address(self) -> str:
        """
        :example: "Departamento 123"
        """
        return self.numerify(self.random_element(self.secondary_address_formats))

    def street_address(self) -> str:
        """
        :example: "Calle 1 N° 23"
        """
        return self.street_name() + " N° " + self.building_number()

    def postcode(self) -> str:
        """
        :example: "1900"
        """
        return self.numerify(self.generator.parse(self.random_element(self.postcode_formats)))

    def address(self) -> str:
        """
        :example: "Calle 1 N° 23, La Plata 1900, Buenos Aires"
        """
        municipality: Tuple[str, str, str] = self.random_element(self.municipalities)
        municipality_code = municipality[0]
        municipality_prov = municipality[2]

        secondary_address: str = self.random_element(
            [
                " " + self.secondary_address(),
                "",
            ]
        )
        postcode = "\n" + municipality[1] + " " + municipality_code
        province_name = ", " + self.provinces[municipality_prov]

        return self.street_address() + secondary_address + postcode + province_name



import datetime as dt
import re

from typing import Pattern

from freezegun import freeze_time

from faker import Faker
from faker.providers.user_agent import Provider as UaProvider


class TestUserAgentProvider:
    """Test user agent provider methods"""

    num_samples = 1000
    android_token_pattern: Pattern = re.compile(r"Android (?P<android_version>\d+(?:\.\d){0,2})")
    ios_token_pattern: Pattern = re.compile(
        r"^(?P<apple_device>.*?); CPU \1 OS " + r"(?P<ios_version>\d+(?:_\d){0,2}) like Mac OS X"
    )
    mac_token_pattern: Pattern = re.compile(r"Macintosh; (?P<mac_processor>.*?) Mac OS X 10_([5-9]|1[0-2])_(\d)")
    one_day = dt.timedelta(1.0)

    def test_android_platform_token(self, faker, num_samples):
        for _ in range(num_samples):
            match = self.android_token_pattern.fullmatch(faker.android_platform_token())
            assert match.group("android_version") in UaProvider.android_versions

    def test_ios_platform_token(self, faker, num_samples):
        for _ in range(num_samples):
            match = self.ios_token_pattern.fullmatch(faker.ios_platform_token())
            assert match.group("apple_device") in UaProvider.apple_devices
            assert match.group("ios_version").replace("_", ".") in UaProvider.ios_versions

    def test_mac_platform_token(self, faker, num_samples):
        for _ in range(num_samples):
            match = self.mac_token_pattern.fullmatch(faker.mac_platform_token())
            assert match.group("mac_processor") in UaProvider.mac_processors

    def test_firefox_deterministic_output(self, faker: Faker, num_samples: int) -> None:
        """Check whether ``faker.firefox()`` is deterministic, given the same seed."""

        for _ in range(num_samples):
            # GIVEN a (new) random seed
            seed = faker.random.random()

            # AND a DevOpsTester using a Faker instance seeded with this seed

            # It is a bit tricky to feed the faker with its own random
            # value, but it is sufficient for this particular test
            faker.seed_instance(seed)

            # AND the DevOpsTester using the fake library tomorrow
            with freeze_time(dt.datetime.now() + self.one_day):
                # AND the DevOpsTester requests a faked Mozilla Firefox web browser user agent (str)
                fake_firefox_ua_output_tomorrow = faker.firefox()

            # WHEN the DevOpsTester would use the fake library with the same seed
            faker.seed_instance(seed)

            # AND the DevOpsTester would use the fake library some time later
            with freeze_time(dt.datetime.max - self.one_day):
                # AND the DevOpsTester requests again faked Mozilla Firefox web browser user agent
                fake_firefox_ua_output_much_later = faker.firefox()

            # THEN the later Firefox U/A output should (always) be equal to the previous one
            assert fake_firefox_ua_output_much_later == fake_firefox_ua_output_tomorrow



from collections import OrderedDict

from .. import Provider as ColorProvider

localized = True


class Provider(ColorProvider):
    """Implement color provider for ``hr_HR`` locale."""

    all_colors = OrderedDict(
        (
            ("Akvamarin", "#7FFFD4"),
            ("Antikna bijela", "#FAEBD7"),
            ("Azurna", "#F0FFFF"),
            ("Bež", "#F5F5DC"),
            ("Bijela", "#FFFFFF"),
            ("Bijelo bilje", "#FFFAF0"),
            ("Bjelokost", "#FFFFF0"),
            ("Blijeda kudelja", "#EEE8AA"),
            ("Blijedi badem", "#FFEBCD"),
            ("Blijedoljubičasta", "#DB7093"),
            ("Blijedotirkizna", "#AFEEEE"),
            ("Blijedozelena", "#98FB98"),
            ("Breskva", "#FFDAB9"),
            ("Brončana", "#D2B48C"),
            ("Čeličnoplava", "#4682B4"),
            ("Čičak", "#D8BFD8"),
            ("Cijan", "#00FFFF"),
            ("Čipka", "#FDF5E6"),
            ("Čokoladna", "#D2691E"),
            ("Crna", "#000000"),
            ("Crvena", "#FF0000"),
            ("Dim", "#F5F5F5"),
            ("Dodger plava", "#1E90FF"),
            ("Duboko ružičasta", "#FF1493"),
            ("Fuksija", "#FF00FF"),
            ("Gainsboro", "#DCDCDC"),
            ("Grimizna", "#DC143C"),
            ("Indigo", "#4B0082"),
            ("Jelenska koža", "#FFE4B5"),
            ("Kadetski plava", "#5F9EA0"),
            ("Kestenjasta", "#800000"),
            ("Koraljna", "#FF7F50"),
            ("Kraljevski plava", "#4169E1"),
            ("Kudelja", "#DAA520"),
            ("Lan", "#FAF0E6"),
            ("Lavanda", "#E6E6FA"),
            ("Limun", "#FFFACD"),
            ("Lipa", "#00FF00"),
            ("Ljubičasta", "#EE82EE"),
            ("Magenta", "#FF00FF"),
            ("Maslinasta", "#808000"),
            ("Medljika", "#F0FFF0"),
            ("Menta", "#F5FFFA"),
            ("Modro nebo", "#00BFFF"),
            ("Modrozelena", "#008080"),
            ("Mornarska", "#000080"),
            ("Morskozelena", "#2E8B57"),
            ("Mračno siva", "#696969"),
            ("Narančasta", "#FFA500"),
            ("Narančastocrvena", "#FF4500"),
            ("Narančastoružičasta", "#FA8072"),
            ("Noćno plava", "#191970"),
            ("Orhideja", "#DA70D6"),
            ("Papaja", "#FFEFD5"),
            ("Peru", "#CD853F"),
            ("Plava", "#0000FF"),
            ("Plavi prah", "#B0E0E6"),
            ("Plavi škriljevac", "#6A5ACD"),
            ("Plavkasta", "#F0F8FF"),
            ("Plavo cvijeće", "#6495ED"),
            ("Plavo nebo", "#87CEEB"),
            ("Plavoljubičasta", "#8A2BE2"),
            ("Porculanska", "#FFE4C4"),
            ("Prljavomaslinasta", "#6B8E23"),
            ("Proljetnozelena", "#00FF7F"),
            ("Prozirno bijela", "#F8F8FF"),
            ("Pšenica", "#F5DEB3"),
            ("Purpurna", "#800080"),
            ("Rajčica", "#FF6347"),
            ("Rumena lavanda", "#FFF0F5"),
            ("Ružičasta", "#FFC0CB"),
            ("Ružičastosmeđa", "#BC8F8F"),
            ("Siva", "#808080"),
            ("Sivi škriljevac", "#708090"),
            ("Sivožuta", "#F0E68C"),
            ("Smeđa", "#A52A2A"),
            ("Smeđe sedlo", "#8B4513"),
            ("Smeđi pijesak", "#F4A460"),
            ("Smeđkasto bijela", "#FFDEAD"),
            ("Snijeg", "#FFFAFA"),
            ("Srebrna", "#C0C0C0"),
            ("Srednja akvamarin", "#66CDAA"),
            ("Srednja crvenoljubičasta", "#C71585"),
            ("Srednja morskozelena", "#3CB371"),
            ("Srednja orhideja", "#BA55D3"),
            ("Srednja plava", "#0000CD"),
            ("Srednja proljetnozelena", "#00FA9A"),
            ("Srednja purpurna", "#9370DB"),
            ("Srednja tirkizna", "#48D1CC"),
            ("Srednje plavi škriljevac", "#7B68EE"),
            ("Svijetla čeličnoplava", "#B0C4DE"),
            ("Svijetla narančastoružičasta", "#FFA07A"),
            ("Svijetli cijan", "#E0FFFF"),
            ("Svijetlo drvo", "#DEB887"),
            ("Svijetlokoraljna", "#F08080"),
            ("Svijetlomorskozelena", "#20B2AA"),
            ("Svijetloplava", "#ADD8E6"),
            ("Svijetloružičasta", "#FFB6C1"),
            ("Svijetlosiva", "#D3D3D3"),
            ("Svijetlosivi škriljevac", "#778899"),
            ("Svijetlozelena", "#90EE90"),
            ("Svijetložuta kudelja", "#FAFAD2"),
            ("Svijetložuta", "#FFFFE0"),
            ("Šamotna opeka", "#B22222"),
            ("Školjka", "#FFF5EE"),
            ("Šljiva", "#DDA0DD"),
            ("Tamna kudelja", "#B8860B"),
            ("Tamna magenta", "#8B008B"),
            ("Tamna narančastoružičasta", "#E9967A"),
            ("Tamna orhideja", "#9932CC"),
            ("Tamna sivožuta", "#BDB76B"),
            ("Tamni cijan", "#008B8B"),
            ("Tamno zelena", "#006400"),
            ("Tamnocrvena", "#8B0000"),
            ("Tamnoljubičasta", "#9400D3"),
            ("Tamnomaslinasta", "#556B2F"),
            ("Tamnonarančasta", "#FF8C00"),
            ("Tamnoplava", "#00008B"),
            ("Tamnoplavi škriljevac", "#483D8B"),
            ("Tamnosiva", "#A9A9A9"),
            ("Tamnosivi škriljevac", "#2F4F4F"),
            ("Tamnotirkizna", "#00CED1"),
            ("Tamnozelena", "#8FBC8F"),
            ("Tirkizna", "#40E0D0"),
            ("Topla ružičasta", "#FF69B4"),
            ("Vedro nebo", "#87CEFA"),
            ("Voda", "#00FFFF"),
            ("Zelena lipa", "#32CD32"),
            ("Zelena šuma", "#228B22"),
            ("Zelena tratina", "#7CFC00"),
            ("Zelena", "#008000"),
            ("Zeleni liker", "#7FFF00"),
            ("Zelenožuta", "#ADFF2F"),
            ("Zlatna", "#FFD700"),
            ("Žućkastocrvena zemlja", "#CD5C5C"),
            ("Žućkastoružičasta", "#FFE4E1"),
            ("Žućkastosmeđa glina", "#A0522D"),
            ("Žuta svila", "#FFF8DC"),
            ("Žuta", "#FFFF00"),
            ("Žutozelena", "#9ACD32"),
        )
    )

    safe_colors = (
        "crna",
        "kestenjasta",
        "zelena",
        "mornarska",
        "maslinasta",
        "purpurna",
        "modrozelena",
        "lipa",
        "plava",
        "srebrna",
        "siva",
        "žuta",
        "fuksija",
        "voda",
        "bijela",
    )



"""
SSN provider for es_MX.

This module adds a provider for mexican SSN, along with Unique Population
Registry Code (CURP) and Federal Taxpayer Registry ID (RFC).
"""

import random
import string

from typing import Optional

from .. import Provider as BaseProvider

try:
    from typing import Literal  # type: ignore
except ImportError:
    from typing_extensions import Literal  # type: ignore


ALPHABET = string.ascii_uppercase
ALPHANUMERIC = string.digits + ALPHABET
VOWELS = "AEIOU"
CONSONANTS = [letter for letter in ALPHABET if letter not in VOWELS]

# https://es.wikipedia.org/wiki/Plantilla:Abreviaciones_de_los_estados_de_M%C3%A9xico
STATES_RENAPO = [
    "AS",
    "BC",
    "BS",
    "CC",
    "CS",
    "CH",
    "DF",
    "CL",
    "CM",
    "DG",
    "GT",
    "GR",
    "HG",
    "JC",
    "MC",
    "MN",
    "MS",
    "NT",
    "NL",
    "OC",
    "PL",
    "QO",
    "QR",
    "SP",
    "SL",
    "SR",
    "TC",
    "TS",
    "TL",
    "VZ",
    "YN",
    "ZS",
    "NE",  # Foreign Born
]

FORBIDDEN_WORDS = {
    "BUEI": "BUEX",
    "BUEY": "BUEX",
    "CACA": "CACX",
    "CACO": "CACX",
    "CAGA": "CAGX",
    "CAGO": "CAGX",
    "CAKA": "CAKX",
    "CAKO": "CAKX",
    "COGE": "COGX",
    "COJA": "COJX",
    "COJE": "COJX",
    "COJI": "COJX",
    "COJO": "COJX",
    "CULO": "CULX",
    "FETO": "FETX",
    "GUEY": "GUEX",
    "JOTO": "JOTX",
    "KACA": "KACX",
    "KACO": "KACX",
    "KAGA": "KAGX",
    "KAGO": "KAGX",
    "KOGE": "KOGX",
    "KOJO": "KOJX",
    "KAKA": "KAKX",
    "KULO": "KULX",
    "MAME": "MAMX",
    "MAMO": "MAMX",
    "MEAR": "MEAX",
    "MEAS": "MEAX",
    "MEON": "MEOX",
    "MION": "MIOX",
    "MOCO": "MOCX",
    "MULA": "MULX",
    "PEDA": "PEDX",
    "PEDO": "PEDX",
    "PENE": "PENX",
    "PUTA": "PUTX",
    "PUTO": "PUTX",
    "QULO": "QULX",
    "RATA": "RATX",
    "RUIN": "RUIN",
}

CURP_CHARACTERS = "0123456789ABCDEFGHIJKLMNÑOPQRSTUVWXYZ"


def _reduce_digits(number: int) -> int:
    """
    Sum of digits of a number until sum becomes single digit.

    Example:
        658 => 6 + 5 + 8 = 19 => 1 + 9 = 10 => 1
    """
    if number == 0:
        return 0
    if number % 9 == 0:
        return 9

    return number % 9


def ssn_checksum(digits: map) -> int:
    """
    Calculate the checksum for the mexican SSN (IMSS).
    """
    return -sum(_reduce_digits(n * (i % 2 + 1)) for i, n in enumerate(digits)) % 10


def curp_checksum(characters: str) -> int:
    """
    Calculate the checksum for the mexican CURP.
    """
    start = 18
    return -sum((start - i) * CURP_CHARACTERS.index(n) for i, n in enumerate(characters)) % 10


class Provider(BaseProvider):
    """
    A Faker provider for the Mexican SSN, RFC and CURP
    """

    ssn_formats = ("###########",)

    def ssn(self) -> str:
        """
        Mexican Social Security Number, as given by IMSS.

        :return: a random Mexican SSN
        """
        office = self.random_int(min=1, max=99)
        birth_year = self.random_int(min=0, max=99)
        start_year = self.random_int(min=0, max=99)
        serial = self.random_int(min=1, max=9999)

        num = f"{office:02d}{start_year:02d}{birth_year:02d}{serial:04d}"

        check = ssn_checksum(map(int, num))
        num += str(check)

        return num

    def curp(self) -> str:
        """
        See https://es.wikipedia.org/wiki/Clave_%C3%9Anica_de_Registro_de_Poblaci%C3%B3n.

        :return: a random Mexican CURP (Unique Population Registry Code)
        """
        birthday = self.generator.date_of_birth()

        first_surname = random.choice(ALPHABET) + random.choice(VOWELS)
        second_surname = random.choice(ALPHABET)
        given_name = random.choice(ALPHABET)
        name_initials = first_surname + second_surname + given_name

        birth_date = birthday.strftime("%y%m%d")
        gender = random.choice("HM")
        state = random.choice(STATES_RENAPO)
        first_surname_inside = random.choice(CONSONANTS)
        second_surname_inside = random.choice(CONSONANTS)
        given_name_inside = random.choice(ALPHABET)

        # This character is assigned to avoid duplicity
        # It's normally '0' for those born < 2000
        # and 'A' for those born >= 2000
        assigned_character = "0" if birthday.year < 2000 else "A"

        name_initials = FORBIDDEN_WORDS.get(name_initials, name_initials)

        random_curp = (
            name_initials
            + birth_date
            + gender
            + state
            + first_surname_inside
            + second_surname_inside
            + given_name_inside
            + assigned_character
        )

        random_curp += str(curp_checksum(random_curp))

        return random_curp

    def rfc(self, natural: bool = True) -> str:
        """
        See https://es.wikipedia.org/wiki/Registro_Federal_de_Contribuyentes

        :param natural: Whether to return the RFC of a natural person.
            Otherwise return the RFC of a legal person.
        :type natural: bool
        :return: a random Mexican RFC
        """
        birthday = self.generator.date_of_birth()

        if natural:
            first_surname = random.choice(ALPHABET) + random.choice(VOWELS)
            second_surname = random.choice(ALPHABET)
            given_name = random.choice(ALPHABET)
            name_initials = first_surname + second_surname + given_name
            name_initials = FORBIDDEN_WORDS.get(name_initials, name_initials)
        else:
            name_initials = (
                self.random_uppercase_letter() + self.random_uppercase_letter() + self.random_uppercase_letter()
            )

        birth_date = birthday.strftime("%y%m%d")
        disambiguation_code = random.choice(ALPHANUMERIC) + random.choice(ALPHANUMERIC) + random.choice(ALPHANUMERIC)

        random_rfc = name_initials + birth_date + disambiguation_code

        return random_rfc

    def elector_code(self, gender: Optional[Literal["H", "M"]] = None) -> str:
        """
        Unique elector code issued by INE (Instituto Nacional Electoral) in Mexico.

        :param gender: Gender for which to generate the code. Will be randomly
            selected if not provided.
        :type gender: str
        :return: a random INE elector code

        :sample:
        :sample: gender='M'
        """
        if gender and gender not in ("H", "M"):
            raise ValueError("Gender must be 'H' or 'M'")

        gender = gender or random.choice(["H", "M"])

        consonants = "".join(random.choices(CONSONANTS, k=6))

        birthday = self.generator.date_of_birth()
        birth_date = birthday.strftime("%y%m%d")

        entity = random.randint(1, 33)
        disambiguation_code = "".join(random.choices(string.digits, k=3))

        return f"{consonants}{birth_date}{entity:02d}{gender}{disambiguation_code}"



from typing import Dict

from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """Implement lorem provider for ``nl_BE`` locale.

    Source: https://nl.wiktionary.org/wiki/WikiWoordenboek:Lijst_met_1000_basiswoorden
    """

    word_list = (
        "aan",
        "aanbod",
        "aanraken",
        "aanval",
        "aap",
        "aardappel",
        "aarde",
        "aardig",
        "acht",
        "achter",
        "actief",
        "activiteit",
        "ademen",
        "af",
        "afgelopen",
        "afhangen",
        "afmaken",
        "afname",
        "afspraak",
        "afval",
        "al",
        "algemeen",
        "alleen",
        "alles",
        "als",
        "alsjeblieft",
        "altijd",
        "ander",
        "andere",
        "anders",
        "angst",
        "antwoord",
        "antwoorden",
        "appel",
        "arm",
        "auto",
        "avond",
        "avondeten",
        "baan",
        "baby",
        "bad",
        "bal",
        "bang",
        "bank",
        "basis",
        "bed",
        "bedekken",
        "bedreiging",
        "bedreven",
        "been",
        "beer",
        "beest",
        "beetje",
        "begin",
        "begrijpen",
        "begrip",
        "behalve",
        "beide",
        "beker",
        "bel",
        "belangrijk",
        "bellen",
        "belofte",
        "beneden",
        "benzine",
        "berg",
        "beroemd",
        "beroep",
        "bescherm",
        "beslissen",
        "best",
        "betalen",
        "beter",
        "bevatten",
        "bewegen",
        "bewolkt",
        "bezoek",
        "bibliotheek",
        "bieden",
        "bij",
        "bijna",
        "bijten",
        "bijvoorbeeld",
        "bijzonder",
        "binnen",
        "binnenkort",
        "blad",
        "blauw",
        "blazen",
        "blij",
        "blijven",
        "bloed",
        "bloem",
        "bodem",
        "boek",
        "boerderij",
        "boete",
        "boom",
        "boon",
        "boord",
        "boos",
        "bord",
        "borstelen",
        "bos",
        "bot",
        "bouwen",
        "boven",
        "branden",
        "brandstof",
        "breed",
        "breken",
        "brengen",
        "brief",
        "broer",
        "broek",
        "brood",
        "brug",
        "bruikbaar",
        "bruiloft",
        "bruin",
        "bui",
        "buiten",
        "bureau",
        "buren",
        "bus",
        "buurman",
        "buurvrouw",
        "cadeau",
        "chocolade",
        "cirkel",
        "comfortabel",
        "compleet",
        "computer",
        "conditie",
        "controle",
        "cool",
        "correct",
        "daar",
        "daarom",
        "dag",
        "dak",
        "dan",
        "dansen",
        "dapper",
        "dat",
        "de",
        "deel",
        "deken",
        "deksel",
        "delen",
        "derde",
        "deze",
        "dichtbij",
        "dienen",
        "diep",
        "dier",
        "dik",
        "ding",
        "dit",
        "dochter",
        "doen",
        "dom",
        "donker",
        "dood",
        "door",
        "doorzichtig",
        "doos",
        "dorp",
        "draad",
        "draaien",
        "dragen",
        "drie",
        "drijven",
        "drinken",
        "drogen",
        "dromen",
        "droog",
        "druk",
        "dubbel",
        "dun",
        "dus",
        "duur",
        "duwen",
        "echt",
        "een",
        "één",
        "eend",
        "eenheid",
        "eenzaam",
        "eerste",
        "eeuw",
        "effect",
        "ei",
        "eigen",
        "eiland",
        "einde",
        "eis",
        "elektrisch",
        "elk",
        "en",
        "enkele",
        "enthousiast",
        "erg",
        "eten",
        "even",
        "examen",
        "extreem",
        "falen",
        "familie",
        "feest",
        "feit",
        "fel",
        "fijn",
        "film",
        "fit",
        "fles",
        "foto",
        "fout",
        "fris",
        "fruit",
        "gaan",
        "gat",
        "gebeuren",
        "gebeurtenis",
        "gebied",
        "geboorte",
        "geboren",
        "gebruik",
        "gebruikelijk",
        "gebruiken",
        "gedrag",
        "gedragen",
        "geel",
        "geen",
        "gehoorzamen",
        "geit",
        "geld",
        "geliefde",
        "gelijk",
        "geloof",
        "geluid",
        "geluk",
        "gemak",
        "gemakkelijk",
        "gemeen",
        "genieten",
        "genoeg",
        "genot",
        "gerecht",
        "gereedschap",
        "geschikt",
        "gespannen",
        "geur",
        "gevaar",
        "gevaarlijk",
        "gevangenis",
        "geven",
        "gevolg",
        "gewicht",
        "gewoon",
        "gezicht",
        "gezond",
        "gif",
        "gisteren",
        "glad",
        "glas",
        "glimlach",
        "god",
        "goed",
        "goedkoop",
        "goud",
        "graf",
        "grap",
        "grappig",
        "gras",
        "grens",
        "grijs",
        "groeien",
        "groen",
        "groente",
        "groep",
        "grof",
        "grond",
        "groot",
        "grootmoeder",
        "grootvader",
        "haan",
        "haar",
        "haast",
        "hal",
        "halen",
        "half",
        "hallo",
        "hamer",
        "hand",
        "hard",
        "hart",
        "haten",
        "hebben",
        "heel",
        "heet",
        "helder",
        "helft",
        "help",
        "hem",
        "hemel",
        "hen",
        "herfst",
        "herinneren",
        "hert",
        "het",
        "heuvel",
        "hier",
        "hij",
        "hobby",
        "hoe",
        "hoed",
        "hoek",
        "hoeveel",
        "hoeveelheid",
        "hoewel",
        "hond",
        "honderd",
        "honger",
        "hoofd",
        "hoog",
        "hoogte",
        "hoop",
        "horen",
        "hotel",
        "houden",
        "huilen",
        "huis",
        "hun",
        "huren",
        "hut",
        "huur",
        "idee",
        "ieder",
        "iedereen",
        "iemand",
        "iets",
        "ijs",
        "ijzer",
        "ik",
        "in",
        "instrument",
        "ja",
        "jaar",
        "jagen",
        "jas",
        "jij",
        "jong",
        "jongen",
        "jouw",
        "jullie",
        "kaars",
        "kaart",
        "kaas",
        "kamer",
        "kans",
        "kant",
        "kantoor",
        "kap",
        "kast",
        "kasteel",
        "kat",
        "kennen",
        "kennis",
        "keuken",
        "keus",
        "kiezen",
        "kijken",
        "kind",
        "kip",
        "kist",
        "klaar",
        "klas",
        "klasse",
        "kleden",
        "klein",
        "kleren",
        "kleur",
        "klimmen",
        "klok",
        "kloppen",
        "klopt",
        "knie",
        "knippen",
        "koers",
        "koffer",
        "koffie",
        "kok",
        "koken",
        "kom",
        "komen",
        "koning",
        "koningin",
        "koorts",
        "kop",
        "kopen",
        "kort",
        "kost",
        "kosten",
        "koud",
        "kraam",
        "kracht",
        "krant",
        "krijgen",
        "kruis",
        "kuil",
        "kunnen",
        "kunst",
        "laag",
        "laat",
        "laatst",
        "lach",
        "lachen",
        "ladder",
        "laken",
        "lamp",
        "land",
        "lang",
        "langs",
        "langzaam",
        "laten",
        "leeftijd",
        "leeg",
        "leerling",
        "leeuw",
        "leger",
        "leiden",
        "lenen",
        "lengte",
        "lepel",
        "leren",
        "les",
        "leuk",
        "leven",
        "lezen",
        "lichaam",
        "licht",
        "liefde",
        "liegen",
        "liggen",
        "lijk",
        "lijken",
        "liniaal",
        "links",
        "lip",
        "list",
        "lomp",
        "lood",
        "lopen",
        "los",
        "lot",
        "lucht",
        "lui",
        "luisteren",
        "lunch",
        "maag",
        "maal",
        "maaltijd",
        "maan",
        "maand",
        "maar",
        "maat",
        "machine",
        "maken",
        "makkelijk",
        "mama",
        "man",
        "mand",
        "manier",
        "map",
        "markeren",
        "markt",
        "me",
        "medicijn",
        "meel",
        "meer",
        "meerdere",
        "meest",
        "meisje",
        "melk",
        "meneer",
        "mengsel",
        "mensen",
        "mes",
        "met",
        "meubel",
        "mevrouw",
        "middel",
        "midden",
        "mij",
        "mijn",
        "miljoen",
        "min",
        "minder",
        "minuut",
        "mis",
        "missen",
        "mits",
        "model",
        "modern",
        "moeder",
        "moeilijk",
        "moeten",
        "mogelijk",
        "mogen",
        "moment",
        "mond",
        "mooi",
        "moord",
        "moorden",
        "morgen",
        "munt",
        "muziek",
        "na",
        "naald",
        "naam",
        "naar",
        "naast",
        "nacht",
        "nat",
        "natuur",
        "natuurlijk",
        "nee",
        "neer",
        "negen",
        "nek",
        "nemen",
        "net",
        "netjes",
        "neus",
        "niet",
        "niets",
        "nieuw",
        "nieuws",
        "nobel",
        "noch",
        "nodig",
        "noemen",
        "nog",
        "nood",
        "nooit",
        "noord",
        "noot",
        "normaal",
        "nu",
        "nul",
        "nummer",
        "object",
        "oceaan",
        "ochtend",
        "oefening",
        "of",
        "offer",
        "olie",
        "olifant",
        "om",
        "oma",
        "onder",
        "onderwerp",
        "onderzoek",
        "oneven",
        "ongeluk",
        "ons",
        "ontsnappen",
        "ontbijt",
        "ontdekken",
        "ontmoeten",
        "ontvangen",
        "ontwikkelen",
        "onze",
        "oog",
        "ooit",
        "ook",
        "oom",
        "oor",
        "oorlog",
        "oorzaak",
        "oost",
        "op",
        "opa",
        "opeens",
        "open",
        "openlijk",
        "opleiding",
        "opnemen",
        "oranje",
        "orde",
        "oud",
        "ouder",
        "over",
        "overal",
        "overeenkomen",
        "overleden",
        "overvallen",
        "paar",
        "paard",
        "pad",
        "pagina",
        "pan",
        "papa",
        "papier",
        "park",
        "partner",
        "pas",
        "passeren",
        "pen",
        "peper",
        "per",
        "perfect",
        "periode",
        "persoon",
        "piano",
        "pijn",
        "pistool",
        "plaat",
        "plaatje",
        "plaats",
        "plafond",
        "plank",
        "plant",
        "plastic",
        "plat",
        "plattegrond",
        "plein",
        "plus",
        "poes",
        "politie",
        "poort",
        "populair",
        "positie",
        "postzegel",
        "potlood",
        "praten",
        "presenteren",
        "prijs",
        "prins",
        "prinses",
        "privé",
        "proberen",
        "probleem",
        "product",
        "provincie",
        "publiek",
        "punt",
        "raak",
        "raam",
        "radio",
        "raken",
        "rapport",
        "recht",
        "rechtdoor",
        "rechts",
        "rechtvaardig",
        "redden",
        "reeds",
        "regen",
        "reiken",
        "reizen",
        "rekenmachine",
        "rennen",
        "repareren",
        "rest",
        "restaurant",
        "resultaat",
        "richting",
        "rijk",
        "rijst",
        "rijzen",
        "ring",
        "rok",
        "rond",
        "rood",
        "rook",
        "rots",
        "roze",
        "rubber",
        "ruiken",
        "ruimte",
        "samen",
        "sap",
        "schaap",
        "schaar",
        "schaduw",
        "scheiden",
        "scherp",
        "schetsen",
        "schieten",
        "schijnen",
        "schip",
        "school",
        "schoon",
        "schouder",
        "schreeuw",
        "schreeuwen",
        "schrijven",
        "schudden",
        "seconde",
        "sex",
        "signaal",
        "simpel",
        "sinds",
        "slaapkamer",
        "slapen",
        "slecht",
        "sleutel",
        "slim",
        "slot",
        "sluiten",
        "smaak",
        "smal",
        "sneeuw",
        "snel",
        "snelheid",
        "snijden",
        "soep",
        "sok",
        "soms",
        "soort",
        "sorry",
        "speciaal",
        "spel",
        "spelen",
        "sport",
        "spreken",
        "springen",
        "staal",
        "stad",
        "stap",
        "start",
        "station",
        "steen",
        "stelen",
        "stem",
        "stempel",
        "ster",
        "sterk",
        "steun",
        "stil",
        "stilte",
        "stoel",
        "stof",
        "stoffig",
        "stom",
        "stop",
        "storm",
        "straat",
        "straffen",
        "structuur",
        "student",
        "studie",
        "stuk",
        "succes",
        "suiker",
        "taal",
        "taart",
        "tafel",
        "tak",
        "tamelijk",
        "tand",
        "tante",
        "tas",
        "taxi",
        "te",
        "team",
        "teen",
        "tegen",
        "teken",
        "tekenen",
        "telefoon",
        "televisie",
        "tellen",
        "tennis",
        "terug",
        "terugkomst",
        "terwijl",
        "test",
        "tevreden",
        "thee",
        "thuis",
        "tien",
        "tijd",
        "titel",
        "toekomst",
        "toen",
        "toename",
        "totaal",
        "traan",
        "tram",
        "trein",
        "trekken",
        "trouwen",
        "trui",
        "tuin",
        "tussen",
        "tweede",
        "u",
        "uit",
        "uitleggen",
        "uitnodigen",
        "uitvinden",
        "uitzoeken",
        "uur",
        "vaak",
        "vaarwel",
        "vader",
        "vak",
        "vakantie",
        "vallen",
        "vals",
        "van",
        "vandaag",
        "vangen",
        "vanmorgen",
        "vannacht",
        "varken",
        "vast",
        "vechten",
        "veel",
        "veer",
        "veilig",
        "ver",
        "veranderen",
        "verandering",
        "verder",
        "verdienen",
        "verdrietig",
        "verenigen",
        "verf",
        "vergelijkbaar",
        "vergelijken",
        "vergelijking",
        "vergeten",
        "vergeven",
        "vergissen",
        "verhaal",
        "verhoging",
        "verjaardag",
        "verkeerd",
        "verkopen",
        "verlaten",
        "verleden",
        "verliezen",
        "vernietigen",
        "veroveren",
        "verrassen",
        "vers",
        "verschil",
        "verschrikkelijk",
        "verspreiden",
        "verstand",
        "verstoppen",
        "versturen",
        "vertellen",
        "vertrekken",
        "vertrouwen",
        "verwachten",
        "verwijderen",
        "verzamelen",
        "verzameling",
        "vet",
        "vier",
        "vierkant",
        "vies",
        "vijand",
        "vijf",
        "vijver",
        "vinden",
        "vinger",
        "vis",
        "vlag",
        "vlees",
        "vlieg",
        "vliegtuig",
        "vloer",
        "voeden",
        "voedsel",
        "voelen",
        "voet",
        "voetbal",
        "vogel",
        "vol",
        "volgende",
        "volgorde",
        "voor",
        "voorbeeld",
        "voorkomen",
        "voorzichtig",
        "voorzien",
        "vork",
        "vorm",
        "vos",
        "vouwen",
        "vraag",
        "vragen",
        "vrede",
        "vreemd",
        "vreemde",
        "vriend",
        "vriendelijk",
        "vriezen",
        "vrij",
        "vrijheid",
        "vroeg",
        "vroeger",
        "vrouw",
        "vullen",
        "vuur",
        "waar",
        "waarom",
        "waarschijnlijk",
        "wachten",
        "wakker",
        "wanneer",
        "want",
        "wapen",
        "warm",
        "wassen",
        "wat",
        "water",
        "we",
        "week",
        "weer",
        "weg",
        "welke",
        "welkom",
        "wens",
        "wereld",
        "werelddeel",
        "werk",
        "west",
        "wetenschap",
        "wie",
        "wiel",
        "wij",
        "wijn",
        "wijs",
        "wild",
        "willen",
        "wind",
        "winkel",
        "winnen",
        "winter",
        "wissen",
        "wit",
        "wolf",
        "wolk",
        "wonder",
        "woord",
        "woud",
        "wreed",
        "zaak",
        "zacht",
        "zak",
        "zand",
        "zee",
        "zeep",
        "zeer",
        "zeggen",
        "zeil",
        "zeker",
        "zelfde",
        "zes",
        "zetten",
        "zeven",
        "ziek",
        "ziekenhuis",
        "ziel",
        "zien",
        "zij",
        "zijn",
        "zilver",
        "zingen",
        "zinken",
        "zitten",
        "zo",
        "zoals",
        "zoeken",
        "zoet",
        "zomer",
        "zon",
        "zonder",
        "zonnig",
        "zoon",
        "zorg",
        "zorgen",
        "zou",
        "zout",
        "zuid",
        "zulke",
        "zullen",
        "zus",
        "zwaar",
        "zwak",
        "zwembad",
        "zwemmen",
    )

    parts_of_speech: Dict[str, tuple] = {}



from .. import Provider as CurrencyProvider


class Provider(CurrencyProvider):
    price_formats = ["###,###,000", "#,###,000,000", "%,###,###,###,###", "%,###,###,###,000,000"]

    def pricetag(self) -> str:
        return self.numerify(self.random_element(self.price_formats)) + "\uFDFC"



import copy
import random
import re

from typing import Pattern

import pytest

from faker.providers.color import RandomColor
from faker.providers.color.az_AZ import Provider as AzAzColorProvider
from faker.providers.color.bg_BG import Provider as BgBgColorProvider
from faker.providers.color.cs_CZ import Provider as CsCzColorProvider
from faker.providers.color.el_GR import Provider as ElGrColorProvider
from faker.providers.color.es_ES import Provider as EsEsColorProvider
from faker.providers.color.fa_IR import Provider as FaIrColorProvider
from faker.providers.color.he_IL import Provider as HeILColorProvider
from faker.providers.color.hy_AM import Provider as HyAmColorProvider
from faker.providers.color.id_ID import Provider as IdIdColorProvider
from faker.providers.color.sk_SK import Provider as SkSkColorProvider


class TestColorProvider:
    """Test color provider methods"""

    num_samples = 10000

    def test_safe_hex_color(self, faker, num_samples):
        assert all(re.fullmatch(r"#(?:([0-9a-f])\1){3}", faker.safe_hex_color()) for _ in range(num_samples))

    def test_hex_color(self, faker, num_samples):
        assert all(re.fullmatch(r"#[0-9a-f]{6}", faker.hex_color()) for _ in range(num_samples))

    def test_rgb_color(self, faker, num_samples):
        for _ in range(num_samples):
            r, g, b = list(map(int, faker.rgb_color().split(",")))
            assert 0 <= r <= 255
            assert 0 <= g <= 255
            assert 0 <= b <= 255

    def test_rgb_css_color(self, faker, num_samples):
        pattern: Pattern = re.compile(r"rgb\((?P<rgb>\d{1,3},\d{1,3},\d{1,3})\)")
        for _ in range(num_samples):
            match = pattern.fullmatch(faker.rgb_css_color())
            rgb = match.group("rgb")
            r, g, b = list(map(int, rgb.split(",")))
            assert 0 <= r <= 255
            assert 0 <= g <= 255
            assert 0 <= b <= 255

    def test_color(self, faker, num_samples):
        random_color = self._seed_instances(faker, 4761)
        expected = [random_color.generate() for _ in range(num_samples)]

        # The `color` provider method should behave like the `generate`
        # method of a standalone RandomColor instance for a given seed
        colors = [faker.color() for _ in range(num_samples)]
        assert colors == expected

    def _seed_instances(self, faker, seed):
        faker.seed_instance(seed)
        return RandomColor(seed=seed)

    def test_color_rgb(self, faker, num_samples):
        random_color = self._seed_instances(faker, 4761)
        expected = [random_color.generate_rgb() for _ in range(num_samples)]
        colors = [faker.color_rgb() for _ in range(num_samples)]
        assert colors == expected

    def test_color_rgb_float(self, faker, num_samples):
        random_color = self._seed_instances(faker, 4761)
        expected = [random_color.generate_rgb_float() for _ in range(num_samples)]
        colors = [faker.color_rgb_float() for _ in range(num_samples)]
        assert colors == expected

    def test_color_hsl(self, faker, num_samples):
        random_color = self._seed_instances(faker, 4761)
        expected = [random_color.generate_hsl() for _ in range(num_samples)]
        colors = [faker.color_hsl() for _ in range(num_samples)]
        assert colors == expected

    def test_color_hsv(self, faker, num_samples):
        random_color = self._seed_instances(faker, 4761)
        expected = [random_color.generate_hsv() for _ in range(num_samples)]
        colors = [faker.color_hsv() for _ in range(num_samples)]
        assert colors == expected


class TestRandomColor:
    """Test RandomColor class"""

    num_samples = 1000
    seed = 4761
    hsv_color_pattern: Pattern = re.compile(
        r"hsv\(" r"(?P<h>\d|[1-9]\d|[1-3]\d{2}), " r"(?P<s>\d|[1-9]\d|100), " r"(?P<v>\d|[1-9]\d|100)\)",
    )
    hsl_color_pattern: Pattern = re.compile(
        r"hsl\(" r"(?P<h>\d|[1-9]\d|[1-3]\d{2}), " r"(?P<s>\d|[1-9]\d|[1-3]\d{2}), " r"(?P<l>\d|[1-9]\d|[1-3]\d{2})\)",
    )
    rgb_color_pattern: Pattern = re.compile(
        r"rgb\(" r"(?P<r>\d|[1-9]\d|[1-3]\d{2}), " r"(?P<g>\d|[1-9]\d|[1-3]\d{2}), " r"(?P<b>\d|[1-9]\d|[1-3]\d{2})\)",
    )
    hex_color_pattern: Pattern = re.compile(r"#[0-9a-f]{6}")

    def setup_method(self):
        self.random_color = RandomColor(seed=self.seed)

    def test_color_format_hsv(self, num_samples):
        for _ in range(num_samples):
            hsv_color = self.random_color.generate(color_format="hsv")
            match = self.hsv_color_pattern.fullmatch(hsv_color)
            assert match
            groupdict = match.groupdict()
            assert 0 <= int(groupdict["h"]) <= 360
            assert 0 <= int(groupdict["s"]) <= 100
            assert 0 <= int(groupdict["v"]) <= 100

    def test_color_format_hsl(self, num_samples):
        for _ in range(num_samples):
            hsl_color = self.random_color.generate(color_format="hsl")
            match = self.hsl_color_pattern.fullmatch(hsl_color)
            assert match
            groupdict = match.groupdict()
            assert 0 <= int(groupdict["h"]) <= 360
            assert 0 <= int(groupdict["s"]) <= 100
            assert 0 <= int(groupdict["l"]) <= 100

    def test_color_format_rgb(self, num_samples):
        for _ in range(num_samples):
            rgb_color = self.random_color.generate(color_format="rgb")
            match = self.rgb_color_pattern.fullmatch(rgb_color)
            assert match
            groupdict = match.groupdict()
            assert 0 <= int(groupdict["r"]) <= 255
            assert 0 <= int(groupdict["g"]) <= 255
            assert 0 <= int(groupdict["b"]) <= 255

    def test_color_format_hex(self, num_samples):
        for _ in range(num_samples):
            hex_color = self.random_color.generate(color_format="hex")
            assert self.hex_color_pattern.fullmatch(hex_color)

    def test_color_format_unspecified(self, num_samples):
        for _ in range(num_samples):
            color = self.random_color.generate()
            assert self.hex_color_pattern.fullmatch(color)

    def test_rgb(self, num_samples):
        for _ in range(num_samples):
            value = self.random_color.generate_rgb()
            assert len(value) == 3
            for i in range(3):
                assert isinstance(value[i], int)
                assert 0 <= value[i] <= 255

    def test_rgb_float(self, num_samples):
        for _ in range(num_samples):
            value = self.random_color.generate_rgb_float()
            assert len(value) == 3
            for i in range(3):
                assert isinstance(value[i], float)
                assert 0 <= value[i] <= 1

    def test_hsl(self, num_samples):
        for _ in range(num_samples):
            value = self.random_color.generate_hsl()
            assert len(value) == 3
            for i in range(3):
                assert isinstance(value[i], int)
            assert 0 <= value[0] <= 360
            assert 0 <= value[1] <= 100
            assert 0 <= value[2] <= 100

    def test_hsv(self, num_samples):
        for _ in range(num_samples):
            value = self.random_color.generate_hsl()
            assert len(value) == 3
            for i in range(3):
                assert isinstance(value[i], int)
            assert 0 <= value[0] <= 360
            assert 0 <= value[1] <= 100
            assert 0 <= value[2] <= 100

    def test_hue_integer(self):
        # HSV format is used, because whatever hue value supplied must be present in the output
        for hue in range(360):
            colors = [self.random_color.generate(hue=hue, color_format="hsv") for _ in range(10)]
            for color in colors:
                match = self.hsv_color_pattern.fullmatch(color)
                assert match
                groupdict = match.groupdict()
                assert int(groupdict["h"]) == hue

    def test_hue_float(self, num_samples):
        baseline_random_color = RandomColor(seed=self.seed)
        for _ in range(num_samples):
            hue_float = random.uniform(0, 360)
            hue_int = int(hue_float)
            expected = [baseline_random_color.generate(hue=hue_int) for _ in range(10)]

            # Using a float value between 0 and 360 should yield the same results
            # as using an integer rounded down from that float value for a given seed
            colors = [self.random_color.generate(hue=hue_float) for _ in range(10)]
            assert colors == expected

    def test_hue_word(self):
        expected = ["#cecece", "#ededed", "#efefef", "#bcbcbc", "#777777"]
        colors = [self.random_color.generate(hue="monochrome") for _ in range(5)]
        assert colors == expected

        expected = ["#ef0b31", "#f2b7ab", "#f74c55", "#a53822", "#8e3712"]
        colors = [self.random_color.generate(hue="red") for _ in range(5)]
        assert colors == expected

        expected = ["#f98313", "#ddb77e", "#f9c413", "#f4ce81", "#ddae71"]
        colors = [self.random_color.generate(hue="orange") for _ in range(5)]
        assert colors == expected

        expected = ["#dbe04e", "#efc621", "#fff65b", "#ceaf27", "#fcf9ae"]
        colors = [self.random_color.generate(hue="yellow") for _ in range(5)]
        assert colors == expected

        expected = ["#05876f", "#57e095", "#50ceaa", "#e4f7a0", "#698909"]
        colors = [self.random_color.generate(hue="green") for _ in range(5)]
        assert colors == expected

        expected = ["#2b839b", "#a4d3e8", "#3d2caa", "#3859a0", "#52349e"]
        colors = [self.random_color.generate(hue="blue") for _ in range(5)]
        assert colors == expected

        expected = ["#a074e8", "#6122bf", "#9f76cc", "#250570", "#3c1599"]
        colors = [self.random_color.generate(hue="purple") for _ in range(5)]
        assert colors == expected

        expected = ["#c605c6", "#fcc4ec", "#d979f7", "#ce108c", "#d3289d"]
        colors = [self.random_color.generate(hue="pink") for _ in range(5)]
        assert colors == expected

    def test_hue_tuple_beyond_limits(self, num_samples):
        baseline_random_color = RandomColor(seed=self.seed)
        expected = [baseline_random_color.generate(hue=[0, 360]) for _ in range(num_samples)]

        # Using a tuple with values not between 0 and 360 should yield the same results
        # as using a tuple with clamped values for a given seed
        colors = [self.random_color.generate(hue=[-100, 4500]) for _ in range(num_samples)]
        assert colors == expected

    def test_hue_tuple_inverted_values(self, num_samples):
        baseline_random_color = RandomColor(seed=self.seed)
        expected = [baseline_random_color.generate(hue=[45, 75]) for _ in range(num_samples)]

        # Using a tuple with inverted values should yield the same results
        # as using the correctly ordered tuple for a given seed
        colors = [self.random_color.generate(hue=[75, 45]) for _ in range(num_samples)]
        assert colors == expected

    def test_hue_invalid(self):
        invalid_values = [
            -0.000000001,  # Very slightly under the min numerical value of 0
            360.000000001,  # Very slightly over the max numerical value of 360
            "invalid value",  # Unsupported string
            [1, 2, 3],  # List with incorrect number of elements of valid data types
            ["ab", 1],  # List with correct number of elements with invalid data types
            self,  # Any other garbage
        ]

        for invalid_value in invalid_values:
            with pytest.raises(TypeError):
                self.random_color.generate(hue=invalid_value)

    def test_luminosity_word(self):
        expected = ["#2b7700", "#073c8c", "#d813aa", "#01961a", "#ce840e"]
        colors = [self.random_color.generate(luminosity="dark") for _ in range(5)]
        assert colors == expected

        expected = ["#16b5ff", "#6266ef", "#fc4e3f", "#b2ff70", "#a30424"]
        colors = [self.random_color.generate(luminosity="bright") for _ in range(5)]
        assert colors == expected

        expected = ["#f276a1", "#fcec94", "#aaffe5", "#ffbd7f", "#98f9dc"]
        colors = [self.random_color.generate(luminosity="light") for _ in range(5)]
        assert colors == expected

        expected = ["#070603", "#99a2a3", "#10a85c", "#3f4f0c", "#004f1c"]
        colors = [self.random_color.generate(luminosity="random") for _ in range(5)]
        assert colors == expected

    def test_luminosity_invalid(self, num_samples):
        baseline_random_color = RandomColor(seed=self.seed)
        expected = [baseline_random_color.generate() for _ in range(num_samples)]

        colors = [self.random_color.generate(luminosity="invalid_value") for _ in range(num_samples)]
        assert colors == expected

    def test_bad_color_map(self):
        # Initial baseline using 62 as hue value
        self.random_color.generate(hue=62)

        # If we remove 62 from the yellow range, calling the previous function should fail
        colormap = copy.deepcopy(self.random_color.colormap)
        colormap["yellow"]["hue_range"] = [(47, 61)]
        self.random_color.colormap = colormap
        with pytest.raises(ValueError):
            self.random_color.generate(hue=62)


class TestAzAz:
    """Test az_AZ color provider methods"""

    def test_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            color_name = faker.color_name()
            assert isinstance(color_name, str)
            assert color_name in AzAzColorProvider.all_colors.keys()


class TestHyAm:
    """Test hy_AM color provider methods"""

    def test_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            color_name = faker.color_name()
            assert isinstance(color_name, str)
            assert color_name in HyAmColorProvider.all_colors.keys()

    def test_safe_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            safe_color_name = faker.safe_color_name()
            assert isinstance(safe_color_name, str)
            assert safe_color_name in HyAmColorProvider.safe_colors


class TestFaIr:
    """Test fa_IR color provider methods"""

    def test_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            color_name = faker.color_name()
            assert isinstance(color_name, str)
            assert color_name in FaIrColorProvider.all_colors.keys()

    def test_safe_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            safe_color_name = faker.safe_color_name()
            assert isinstance(safe_color_name, str)
            assert safe_color_name in FaIrColorProvider.safe_colors


class TestBgBg:
    """Test bg_BG color provider methods"""

    def test_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            color_name = faker.color_name()
            assert isinstance(color_name, str)
            assert color_name in BgBgColorProvider.all_colors.keys()

    def test_safe_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            safe_color_name = faker.safe_color_name()
            assert isinstance(safe_color_name, str)
            assert safe_color_name in BgBgColorProvider.safe_colors


class TestElGr:
    """Test el_GR color provider methods"""

    def test_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            color_name = faker.color_name()
            assert isinstance(color_name, str)
            assert color_name in ElGrColorProvider.all_colors.keys()

    def test_safe_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            safe_color_name = faker.safe_color_name()
            assert isinstance(safe_color_name, str)
            assert safe_color_name in ElGrColorProvider.safe_colors


class TestEsEs:
    """Test es_ES color provider methods"""

    def test_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            color_name = faker.color_name()
            assert isinstance(color_name, str)
            assert color_name in EsEsColorProvider.all_colors.keys()

    def test_safe_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            safe_color_name = faker.safe_color_name()
            assert isinstance(safe_color_name, str)
            assert safe_color_name in EsEsColorProvider.safe_colors


class TestSkSk:
    """Test sk_SK color provider methods"""

    def test_safe_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            safe_color_name = faker.safe_color_name()
            assert isinstance(safe_color_name, str)
            assert safe_color_name in SkSkColorProvider.safe_colors


class TestCsCz:
    """Test cs_CZ color provider methods"""

    def test_safe_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            safe_color_name = faker.safe_color_name()
            assert isinstance(safe_color_name, str)
            assert safe_color_name in CsCzColorProvider.safe_colors


class TestHeIl:
    """Test he_IL color provider methods"""

    def test_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            color_name = faker.color_name()
            assert isinstance(color_name, str)
            assert color_name in HeILColorProvider.all_colors.keys()

    def test_safe_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            safe_color_name = faker.safe_color_name()
            assert isinstance(safe_color_name, str)
            assert safe_color_name in HeILColorProvider.safe_colors


class TestIdId:
    """Test id_ID color provider methods"""

    def test_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            color_name = faker.color_name()
            assert isinstance(color_name, str)
            assert color_name in IdIdColorProvider.all_colors.keys()

    def test_safe_color_name(self, faker, num_samples):
        for _ in range(num_samples):
            safe_color_name = faker.safe_color_name()
            assert isinstance(safe_color_name, str)
            assert safe_color_name in IdIdColorProvider.safe_colors



#  tests for faker-24.14.1-py311h06a4308_0 (this is a generated file);
print('===== testing package: faker-24.14.1-py311h06a4308_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
import faker
import faker.providers
import faker.providers.address
import faker.providers.address.cs_CZ
import faker.providers.address.de_DE
import faker.providers.address.el_GR
import faker.providers.address.en
import faker.providers.address.en_AU
import faker.providers.address.en_CA
import faker.providers.address.en_GB
import faker.providers.address.en_US
import faker.providers.address.es
import faker.providers.address.es_ES
import faker.providers.address.es_MX
import faker.providers.address.fa_IR
import faker.providers.address.fi_FI
import faker.providers.address.fr_CH
import faker.providers.address.fr_FR
import faker.providers.address.hi_IN
import faker.providers.address.hr_HR
import faker.providers.address.it_IT
import faker.providers.address.ja_JP
import faker.providers.address.ko_KR
import faker.providers.address.ne_NP
import faker.providers.address.nl_BE
import faker.providers.address.nl_NL
import faker.providers.address.no_NO
import faker.providers.address.pl_PL
import faker.providers.address.pt_BR
import faker.providers.address.pt_PT
import faker.providers.address.ru_RU
import faker.providers.address.sk_SK
import faker.providers.address.sl_SI
import faker.providers.address.sv_SE
import faker.providers.address.uk_UA
import faker.providers.address.zh_CN
import faker.providers.address.zh_TW
import faker.providers.barcode
import faker.providers.barcode.en_US
import faker.providers.color
import faker.providers.color.en_US
import faker.providers.color.uk_UA
import faker.providers.company
import faker.providers.company.bg_BG
import faker.providers.company.cs_CZ
import faker.providers.company.de_DE
import faker.providers.company.en_US
import faker.providers.company.es_MX
import faker.providers.company.fa_IR
import faker.providers.company.fi_FI
import faker.providers.company.fr_CH
import faker.providers.company.fr_FR
import faker.providers.company.hr_HR
import faker.providers.company.it_IT
import faker.providers.company.ja_JP
import faker.providers.company.ko_KR
import faker.providers.company.no_NO
import faker.providers.company.pt_BR
import faker.providers.company.pt_PT
import faker.providers.company.ru_RU
import faker.providers.company.sk_SK
import faker.providers.company.sl_SI
import faker.providers.company.sv_SE
import faker.providers.company.zh_CN
import faker.providers.company.zh_TW
import faker.providers.credit_card
import faker.providers.credit_card.en_US
import faker.providers.currency
import faker.providers.currency.en_US
import faker.providers.date_time
import faker.providers.date_time.en_US
import faker.providers.file
import faker.providers.file.en_US
import faker.providers.internet
import faker.providers.internet.bg_BG
import faker.providers.internet.bs_BA
import faker.providers.internet.cs_CZ
import faker.providers.internet.de_AT
import faker.providers.internet.de_DE
import faker.providers.internet.el_GR
import faker.providers.internet.en_AU
import faker.providers.internet.en_US
import faker.providers.internet.fa_IR
import faker.providers.internet.fi_FI
import faker.providers.internet.fr_CH
import faker.providers.internet.fr_FR
import faker.providers.internet.hr_HR
import faker.providers.internet.ja_JP
import faker.providers.internet.ko_KR
import faker.providers.internet.no_NO
import faker.providers.internet.pt_BR
import faker.providers.internet.pt_PT
import faker.providers.internet.ru_RU
import faker.providers.internet.sk_SK
import faker.providers.internet.sl_SI
import faker.providers.internet.sv_SE
import faker.providers.internet.uk_UA
import faker.providers.internet.zh_CN
import faker.providers.job
import faker.providers.job.en_US
import faker.providers.job.fa_IR
import faker.providers.job.fr_CH
import faker.providers.job.fr_FR
import faker.providers.job.hr_HR
import faker.providers.job.pl_PL
import faker.providers.job.ru_RU
import faker.providers.job.uk_UA
import faker.providers.job.zh_TW
import faker.providers.lorem
import faker.providers.lorem.el_GR
import faker.providers.lorem.la
import faker.providers.lorem.ru_RU
import faker.providers.misc
import faker.providers.misc.en_US
import faker.providers.person
import faker.providers.person.bg_BG
import faker.providers.person.cs_CZ
import faker.providers.person.de_AT
import faker.providers.person.de_DE
import faker.providers.person.da_DK
import faker.providers.person.el_GR
import faker.providers.person.en
import faker.providers.person.en_GB
import faker.providers.person.en_US
import faker.providers.person.es_ES
import faker.providers.person.es_MX
import faker.providers.person.fa_IR
import faker.providers.person.fi_FI
import faker.providers.person.fr_CH
import faker.providers.person.fr_FR
import faker.providers.person.hi_IN
import faker.providers.person.hr_HR
import faker.providers.person.it_IT
import faker.providers.person.ja_JP
import faker.providers.person.ko_KR
import faker.providers.person.lt_LT
import faker.providers.person.lv_LV
import faker.providers.person.ne_NP
import faker.providers.person.nl_NL
import faker.providers.person.no_NO
import faker.providers.person.pl_PL
import faker.providers.person.pt_BR
import faker.providers.person.pt_PT
import faker.providers.person.ru_RU
import faker.providers.person.sl_SI
import faker.providers.person.sv_SE
import faker.providers.person.tr_TR
import faker.providers.person.uk_UA
import faker.providers.person.zh_CN
import faker.providers.person.zh_TW
import faker.providers.phone_number
import faker.providers.phone_number.bg_BG
import faker.providers.phone_number.bs_BA
import faker.providers.phone_number.cs_CZ
import faker.providers.phone_number.de_DE
import faker.providers.phone_number.da_DK
import faker.providers.phone_number.el_GR
import faker.providers.phone_number.en_AU
import faker.providers.phone_number.en_CA
import faker.providers.phone_number.en_GB
import faker.providers.phone_number.en_US
import faker.providers.phone_number.es_ES
import faker.providers.phone_number.es_MX
import faker.providers.phone_number.fa_IR
import faker.providers.phone_number.fi_FI
import faker.providers.phone_number.fr_CH
import faker.providers.phone_number.fr_FR
import faker.providers.phone_number.hi_IN
import faker.providers.phone_number.hr_HR
import faker.providers.phone_number.it_IT
import faker.providers.phone_number.ja_JP
import faker.providers.phone_number.ko_KR
import faker.providers.phone_number.lt_LT
import faker.providers.phone_number.lv_LV
import faker.providers.phone_number.ne_NP
import faker.providers.phone_number.nl_BE
import faker.providers.phone_number.nl_NL
import faker.providers.phone_number.no_NO
import faker.providers.phone_number.pl_PL
import faker.providers.phone_number.pt_BR
import faker.providers.phone_number.pt_PT
import faker.providers.phone_number.ru_RU
import faker.providers.phone_number.sk_SK
import faker.providers.phone_number.sl_SI
import faker.providers.phone_number.sv_SE
import faker.providers.phone_number.tr_TR
import faker.providers.phone_number.uk_UA
import faker.providers.phone_number.zh_CN
import faker.providers.phone_number.zh_TW
import faker.providers.profile
import faker.providers.profile.en_US
import faker.providers.python
import faker.providers.python.en_US
import faker.providers.ssn
import faker.providers.ssn.en_CA
import faker.providers.ssn.en_US
import faker.providers.ssn.fi_FI
import faker.providers.ssn.fr_CH
import faker.providers.ssn.hr_HR
import faker.providers.ssn.it_IT
import faker.providers.ssn.ko_KR
import faker.providers.ssn.nl_BE
import faker.providers.ssn.nl_NL
import faker.providers.ssn.pt_BR
import faker.providers.ssn.ru_RU
import faker.providers.ssn.sv_SE
import faker.providers.ssn.uk_UA
import faker.providers.ssn.zh_CN
import faker.providers.ssn.zh_TW
import faker.providers.user_agent
import faker.providers.user_agent.en_US
import faker.utils
#  --- run_test.py (end) ---

print('===== faker-24.14.1-py311h06a4308_0 OK =====');
print("import: 'faker'")
import faker

print("import: 'faker.cli'")
import faker.cli

print("import: 'faker.documentor'")
import faker.documentor

print("import: 'faker.exceptions'")
import faker.exceptions

print("import: 'faker.factory'")
import faker.factory

print("import: 'faker.generator'")
import faker.generator

print("import: 'faker.proxy'")
import faker.proxy

